@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@ import java.rmi.RemoteException;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.MissingOptionException;
 import org.apache.commons.cli.Options;
+import com.openexchange.auth.Credentials;
 import com.openexchange.auth.rmi.RemoteAuthenticator;
 import com.openexchange.cli.AbstractRmiCLI;
 import com.openexchange.gdpr.dataexport.rmi.DataExportRMIService;
@@ -103,12 +104,15 @@ public class DataExportCancelCLT extends AbstractRmiCLI<Void> {
             throw new MissingOptionException("Please specify a valid context identifier");
         }
 
+        String adminUser = cmd.getOptionValue('A');
+        String adminPassword = cmd.getOptionValue('P');
+
         DataExportRMIService rmiService = getRmiStub(optRmiHostName, DataExportRMIService.RMI_NAME);
         if (userId > 0) {
-            rmiService.cancelDataExportTask(userId, contextId);
+            rmiService.cancelDataExportTask(userId, contextId, new Credentials(adminUser, adminPassword));
             System.out.println("Requested to cancel data export for user " + userId + " in context " + contextId);
         } else {
-            rmiService.cancelDataExportTasks(contextId);
+            rmiService.cancelDataExportTasks(contextId, new Credentials(adminUser, adminPassword));
             System.out.println("Requested to cancel data export for context " + contextId);
         }
         return null;
