@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -35,6 +35,7 @@ import org.apache.commons.cli.MissingOptionException;
 import org.apache.commons.cli.Option;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.ParseException;
+import com.openexchange.auth.Credentials;
 import com.openexchange.auth.rmi.RemoteAuthenticator;
 import com.openexchange.cli.AbstractRmiCLI;
 import com.openexchange.gdpr.dataexport.rmi.DataExportRMIService;
@@ -88,8 +89,11 @@ public class DataExportConsistencyCLT extends AbstractRmiCLI<Void> {
 
     @Override
     protected Void invoke(Options options, CommandLine cmd, String optRmiHostName) throws Exception {
+        String adminUser = cmd.getOptionValue('A');
+        String adminPassword = cmd.getOptionValue('P');
+
         if (repair) {
-            return repair(optRmiHostName);
+            return repair(optRmiHostName, adminUser, adminPassword);
         }
 
         return list(optRmiHostName);
@@ -139,9 +143,9 @@ public class DataExportConsistencyCLT extends AbstractRmiCLI<Void> {
         return Boolean.TRUE;
     }
 
-    private Void repair(String optRmiHostName) throws MalformedURLException, RemoteException, NotBoundException {
+    private Void repair(String optRmiHostName, String adminUser, String adminPassword) throws MalformedURLException, RemoteException, NotBoundException {
         DataExportRMIService rmiService = getRmiStub(optRmiHostName, DataExportRMIService.RMI_NAME);
-        int fixedEntries = rmiService.fixOrphanedEntries(filestoreIds);
+        int fixedEntries = rmiService.fixOrphanedEntries(filestoreIds, new Credentials(adminUser, adminPassword));
         System.out.println("Fixed a total of " + fixedEntries + " entries.");
         return null;
     }
