@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -32,7 +32,6 @@ import org.apache.commons.cli.Options;
 import com.openexchange.auth.rmi.RemoteAuthenticator;
 import com.openexchange.cli.AbstractRmiCLI;
 import com.openexchange.gdpr.dataexport.rmi.DataExportRMIService;
-import com.openexchange.java.Strings;
 import com.openexchange.tools.console.TableWriter;
 import com.openexchange.tools.console.TableWriter.ColumnFormat;
 import com.openexchange.tools.console.TableWriter.ColumnFormat.Align;
@@ -75,21 +74,6 @@ public class DataExportListCLT extends AbstractRmiCLI<Void> {
 
     @Override
     protected void administrativeAuth(String login, String password, CommandLine cmd, RemoteAuthenticator authenticator) throws RemoteException {
-        // Try context administrator authentication first
-        if (contextId > 0) {
-            try {
-                authenticator.doAuthentication(login, password, contextId);
-                return;
-            } catch (RemoteException e) {
-                if (e.getMessage() == null || Strings.asciiLowerCase(e.getMessage()).indexOf("authentication failed") < 0) {
-                    throw e;
-                }
-                // Context administrator authentication failed. Try with master authentication
-            }
-        }
-
-        // Master authentication
-        authenticator.doAuthentication(login, password);
     }
 
     @Override
