@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -79,6 +79,10 @@ import com.openexchange.gdpr.dataexport.TimeRange;
 import com.openexchange.gdpr.dataexport.impl.notification.DataExportNotificationSender;
 import com.openexchange.gdpr.dataexport.impl.notification.Reason;
 import com.openexchange.java.ISO8601Utils;
+import com.openexchange.java.MessageFormatter;
+import com.openexchange.java.Strings;
+import com.openexchange.logging.Constants;
+import com.openexchange.logging.LogUtility;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 import com.openexchange.threadpool.AbstractTask;
@@ -150,7 +154,7 @@ public class DataExportServiceImpl implements DataExportService {
                             Optional<DataExportStatus> optionalStatus = storageService.getDataExportStatus(task.getUserId(), task.getContextId());
                             if (optionalStatus.isPresent() && optionalStatus.get().isAborted()) {
                                 // Task has been aborted
-                                execution.stop();
+                                execution.stop(MessageFormatter.format("Data export task {} of user {} in context {} has been marked as aborted", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId())));
                             }
                         } catch (Exception e) {
                             LOG.warn("Failed to retrieve status for data export task.", e);
@@ -196,10 +200,17 @@ public class DataExportServiceImpl implements DataExportService {
                             if (Reason.SUCCESS == reason) {
                                 try {
                                     Optional<Date> lastAccessedTimeStamp = storageService.getLastAccessedTimeStamp(userId, contextId);
-                                    expiryDate = new Date(lastAccessedTimeStamp.get().getTime() + config.getMaxTimeToLiveMillis());
+                                    if (lastAccessedTimeStamp.isPresent()) {
+                                        expiryDate = new Date(lastAccessedTimeStamp.get().getTime() + config.getMaxTimeToLiveMillis());
+                                    } else {
+                                        Date fallback = new Date(System.currentTimeMillis() + config.getMaxTimeToLiveMillis());
+                                        expiryDate = fallback;
+                                        LOG.warn("No such last-accessed time stamp for data export task {} of user {} in context {}. Assuming \"{}\" as expiration date.", stringFor(taskId), I(userId), I(contextId), LogUtility.toStringObjectFor(() -> ISO8601Utils.format(fallback)));
+                                    }
                                 } catch (Exception e) {
-                                    expiryDate = new Date(System.currentTimeMillis() + config.getMaxTimeToLiveMillis());
-                                    LOG.warn("Failed to query last-accessed time stamp from data export task {} of user {} in context {}. Assuming \"{}\" as expiration date.", stringFor(taskId), I(userId), I(contextId), ISO8601Utils.format(expiryDate), e);
+                                    Date fallback = new Date(System.currentTimeMillis() + config.getMaxTimeToLiveMillis());
+                                    expiryDate = fallback;
+                                    LOG.warn("Failed to query last-accessed time stamp from data export task {} of user {} in context {}. Assuming \"{}\" as expiration date.", stringFor(taskId), I(userId), I(contextId), ISO8601Utils.format(fallback), e);
                                 }
                                 try {
                                     Optional<DataExportTask> optionalTask = storageService.getDataExportTask(userId, contextId);
@@ -225,6 +236,8 @@ public class DataExportServiceImpl implements DataExportService {
         };
         long delayMillis = config.getCheckForAbortedTasksFrequency();
         checkAbortedTimerTask = timerService.scheduleAtFixedRate(abortedChecker, delayMillis, delayMillis, TimeUnit.MILLISECONDS);
+
+        LOG.info("Initiated data export service checking for tasks every {}msec utilizing:{}    {}", L(config.getCheckForTasksFrequency()), Strings.getLineSeparator(), config.getRangesOfTheWeek());
     }
 
     @Override
@@ -244,7 +257,7 @@ public class DataExportServiceImpl implements DataExportService {
 
     @Override
     public List<DataExportTask> getDataExportTasks(boolean checkValidity) throws OXException {
-        if (checkValidity == false) {
+        if (!checkValidity) {
             return storageService.getDataExportTasks();
         }
 
@@ -257,7 +270,7 @@ public class DataExportServiceImpl implements DataExportService {
         if (contextService != null) {
             UserService userService = services.getOptionalService(UserService.class);
             if (userService != null) {
-                dataExportTasks = checkTasksValidity(dataExportTasks, userService, contextService);
+                dataExportTasks = checkTasksValidity(dataExportTasks, userService, contextService); // NOSONARLINT
             }
         }
 
@@ -279,7 +292,7 @@ public class DataExportServiceImpl implements DataExportService {
             }
 
             if (exists.booleanValue()) {
-                if (userExists(dataExportTask.getUserId(), contextId, userService) == false) {
+                if (!userExists(dataExportTask.getUserId(), contextId, userService)) {
                     // Remove task
                     it.remove();
                     if (tasksToDelete == null) {
@@ -306,7 +319,7 @@ public class DataExportServiceImpl implements DataExportService {
                 public Void call() throws Exception {
                     for (DataExportTask task : tasks) {
                         storageService.deleteDataExportTask(task.getId());
-                        LOG.info("Deleted data export task {} of user {} in context {} since associated user does not exist.", DataExportUtility.stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                        LOG.info("Deleted data export task {} of user {} in context {} since associated user does not exist.", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
                     }
                     return null;
                 }
@@ -352,15 +365,16 @@ public class DataExportServiceImpl implements DataExportService {
             throw DataExportExceptionCode.NO_MODULES_SPECIFIED.create();
         }
 
-        int fileStorageId;
-        {
-            ConfigViewFactory viewFactory = services.getServiceSafe(ConfigViewFactory.class);
-            ConfigView view = viewFactory.getView(userId, contextId);
+        int fileStorageId = ConfigViews.getDefinedIntPropertyFrom("com.openexchange.gdpr.dataexport.fileStorageId", -1, services.getServiceSafe(ConfigViewFactory.class).getView(userId, contextId));
+        if (fileStorageId <= 0) {
+            throw DataExportExceptionCode.NO_FILE_STORAGE_SPECIFIED.create();
+        }
 
-            fileStorageId = ConfigViews.getDefinedIntPropertyFrom("com.openexchange.gdpr.dataexport.fileStorageId", -1, view);
-            if (fileStorageId <= 0) {
-                throw DataExportExceptionCode.NO_FILE_STORAGE_SPECIFIED.create();
-            }
+        // Check availability
+        try {
+            DataExportUtility.getFileStorageFor(fileStorageId, contextId);
+        } catch (OXException e) {
+            throw DataExportExceptionCode.FILE_STORAGE_INACCESSIBLE.create(e, I(fileStorageId), I(userId), I(contextId));
         }
 
         List<DataExportWorkItem> workItems = new ArrayList<>(numberOfModulesToExport);
@@ -413,7 +427,7 @@ public class DataExportServiceImpl implements DataExportService {
                     }
                 }
             } catch (Exception e) {
-                LOG.error("Failed checking for existent aborted data export task of user {} in context {}", I(task.getUserId()), I(contextId));
+                LOG.error("Failed checking for existent aborted data export task of user {} in context {}", I(task.getUserId()), I(contextId), e);
             }
         }
         return Optional.empty();
@@ -456,12 +470,18 @@ public class DataExportServiceImpl implements DataExportService {
             for (DataExportTaskExecution execution : executions.values()) {
                 optionalTask = execution.getCurrentTask();
                 if (optionalTask.isPresent() && equals(task, optionalTask.get())) {
-                    execution.stop();
+                    if (task != null) { // NOSONARLINT
+                        execution.stop(MessageFormatter.format("Data export task {} of user {} in context {} has been requested for being canceled", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId())));
+                    }
                 }
             }
         } catch (Exception e) {
             // Stop attempt failed
-            LOG.warn("Failed to locally stop task {} from user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
+            if (task == null) {
+                LOG.warn("Failed to locally stop task", e);
+            } else {
+                LOG.warn("Failed to locally stop task {} from user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
+            }
         }
     }
 
@@ -539,6 +559,7 @@ public class DataExportServiceImpl implements DataExportService {
 
         // Check task's status
         DataExportTask task = optionalTask.get();
+        checkAndRepairStatus(task);
         DataExportStatus status = task.getStatus();
         if (!status.isDone()) {
             return Optional.of(DefaultDataExport.builder().withTask(task).build());
@@ -599,8 +620,7 @@ public class DataExportServiceImpl implements DataExportService {
             throw DataExportExceptionCode.TASK_NOT_COMPLETED.create(I(userId), I(contextId));
         }
 
-        FileLocations fileLocations = optionalResultFiles.get();
-        List<FileLocation> locations = fileLocations.getLocations();
+        List<FileLocation> locations = optionalResultFiles.get().getLocations();
         FileLocation fileLocation = null;
         for (Iterator<FileLocation> it = locations.iterator(); fileLocation == null && it.hasNext(); ) {
             FileLocation loc = it.next();
@@ -614,8 +634,7 @@ public class DataExportServiceImpl implements DataExportService {
         }
 
         // User
-        UserService userService = services.getServiceSafe(UserService.class);
-        User user = userService.getUser(userId, contextId);
+        User user = services.getServiceSafe(UserService.class).getUser(userId, contextId);
 
         // File name
         String fileName = DataExportUtility.generateFileNameFor("archive", ".zip", number, locations.size(), task.getCreationTime(), user);
@@ -971,86 +990,124 @@ public class DataExportServiceImpl implements DataExportService {
     }
 
     synchronized void startProcessingTasks() throws OXException {
-        // Grab services
-        ThreadPoolService threadPool = services.getServiceSafe(ThreadPoolService.class);
-        DatabaseService databaseService = services.getServiceSafe(DatabaseService.class);
-
-        DataExportLock lock = DataExportLock.getInstance();
-        DataExportLock.LockAcquisition acquisition = lock.acquireCleanUpTaskLock(false, databaseService);
+        LOG.info(Constants.DROP_MDC_MARKER, "Checking for data export tasks to start");
         try {
-            if (acquisition.isNotAcquired()) {
-                // Failed to acquire clean-up lock
-                LOG.info("Failed to acquire clean-up lock for data export files. Skipping job execution run...");
-                return;
-            }
+            // Grab services
+            ThreadPoolService threadPool = services.getServiceSafe(ThreadPoolService.class);
+            DatabaseService databaseService = services.getServiceSafe(DatabaseService.class);
 
-            // Obtain and check executions mapping
-            final Map<Future<Void>, DataExportTaskExecution> executions = this.executions;
-            for (Iterator<Entry<Future<Void>, DataExportTaskExecution>> it = executions.entrySet().iterator(); it.hasNext();) {
-                Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry = it.next();
-                DataExportTaskExecution execution = executionEntry.getValue();
-                if (execution.isInvalid()) {
-                    it.remove();
-                    execution.stop();
-                    executionEntry.getKey().cancel(true);
-                }
-            }
-
-            // Start new executions as needed
-            int count = 1;
-            while (executions.size() < config.getNumberOfConcurrentTasks()) {
-                Optional<DataExportJob> dataExportJob = storageService.getNextDataExportJob();
-                if (!dataExportJob.isPresent()) {
-                    // No pending/paused/expired tasks available
-                    LOG.debug("Currently there are no data export tasks to execute");
+            DataExportLock lock = DataExportLock.getInstance();
+            DataExportLock.LockAcquisition acquisition = lock.acquireCleanUpTaskLock(false, databaseService);
+            try {
+                if (acquisition.isNotAcquired()) {
+                    // Failed to acquire clean-up lock
+                    LOG.info(Constants.DROP_MDC_MARKER, "Failed to acquire clean-up lock for data export files. Skipping job execution run...");
                     return;
                 }
 
-                // Get job
-                DataExportJob job = dataExportJob.get();
+                // Obtain and check executions mapping
+                LOG.info(Constants.DROP_MDC_MARKER, "Acquired clean-up lock for data export files. Initiating job execution run...");
+                final Map<Future<Void>, DataExportTaskExecution> executions = this.executions;
+                for (Iterator<Entry<Future<Void>, DataExportTaskExecution>> it = executions.entrySet().iterator(); it.hasNext();) {
+                    Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry = it.next();
+                    DataExportTaskExecution execution = executionEntry.getValue();
+                    if (execution.isInvalid()) {
+                        it.remove();
+                        execution.stop("Processing thread no more alive");
+                        executionEntry.getKey().cancel(true);
 
-                // Artificial delay
-                long nanosToWait = TimeUnit.NANOSECONDS.convert((count++ * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
-                LockSupport.parkNanos(nanosToWait);
-
-                // Diagnostics report
-                DiagnosticsReportOptions reportOptions = DiagnosticsReportOptions.builder()
-                    .withAddDiagnosticsReport(isAddDiagnosticsReport(job))
-                    .withConsiderPermissionDeniedErrors(isConsiderPermissionDeniedErrors(job))
-                    .build();
-
-                // Some variables for clean-up
-                DataExportTaskExecution execution = null;
-                Future<Void> future = null;
-                boolean openedForProcessing = false;
-                try {
-                    // Initialize execution & submit it to thread pool
-                    execution = new DataExportTaskExecution(job, reportOptions, config, storageService, providerRegistry, services);
-                    future = threadPool.submit(execution);
-
-                    // Store execution in map. Then open it for being processed while registering a clean-up task that ensures execution is removed from map when finished
-                    executions.put(future, execution);
-                    execution.allowProcessing(new AllowProcessingRunnable(future, executions));
-                    openedForProcessing = true;
-                } finally {
-                    if (!openedForProcessing) {
-                        if (execution != null) {
-                            execution.stop();
+                        Optional<DataExportTask> optTask = executionEntry.getValue().getCurrentTask();
+                        if (optTask.isPresent()) {
+                            DataExportTask task = optTask.get();
+                            LOG.debug(Constants.DROP_MDC_MARKER, "Canceled invalid job execution for data export task {} of user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
                         }
-
-                        if (future != null) {
-                            executions.remove(future);
-                            future.cancel(true);
+                    } else {
+                        Optional<DataExportTask> optTask = executionEntry.getValue().getCurrentTask();
+                        if (optTask.isPresent()) {
+                            DataExportTask task = optTask.get();
+                            LOG.debug(Constants.DROP_MDC_MARKER, "Found running job execution for data export task {} of user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
                         }
                     }
                 }
+
+                // Check if a new execution may be spawned
+                if (executions.size() >= config.getNumberOfConcurrentTasks()) {
+                    if (executions.size() == 1) {
+                        LOG.debug(Constants.DROP_MDC_MARKER, "There is already a data export tasks in execution. Leaving since there may not be more than {} in concurrent execution", I(config.getNumberOfConcurrentTasks()));
+                    } else {
+                        LOG.debug(Constants.DROP_MDC_MARKER, "There are already {} data export tasks in execution. Leaving since there may not be more than {} in concurrent execution", I(executions.size()), I(config.getNumberOfConcurrentTasks()));
+                    }
+                    return;
+                }
+
+                // Start new executions as needed
+                int count = 1;
+                do {
+                    Optional<DataExportJob> dataExportJob = storageService.getNextDataExportJob();
+                    if (!dataExportJob.isPresent()) {
+                        // No pending/paused/expired tasks available
+                        LOG.debug(Constants.DROP_MDC_MARKER, "Currently there are no data export tasks to execute");
+                        return;
+                    }
+
+                    // Get job
+                    DataExportJob job = dataExportJob.get();
+                    LOG.debug(Constants.DROP_MDC_MARKER, "Got next data export task {} of user {} in context {} for being executed", stringFor(job.getDataExportTask().getId()), I(job.getDataExportTask().getUserId()), I(job.getDataExportTask().getContextId()));
+
+                    // Artificial delay
+                    long nanosToWait = TimeUnit.NANOSECONDS.convert((count++ * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS); // NOSONARLINT
+                    LockSupport.parkNanos(nanosToWait);
+
+                    // Diagnostics report
+                    DiagnosticsReportOptions reportOptions = DiagnosticsReportOptions.builder()
+                        .withAddDiagnosticsReport(isAddDiagnosticsReport(job))
+                        .withConsiderPermissionDeniedErrors(isConsiderPermissionDeniedErrors(job))
+                        .build();
+
+                    // Some variables for clean-up
+                    DataExportTaskExecution execution = null;
+                    Future<Void> future = null;
+                    boolean openedForProcessing = false;
+                    try {
+                        // Initialize execution & submit it to thread pool
+                        execution = new DataExportTaskExecution(job, reportOptions, config, storageService, providerRegistry, services);
+                        future = threadPool.submit(execution);
+                        LOG.debug(Constants.DROP_MDC_MARKER, "Submitted execution for data export task {} of user {} in context {} with executor {}", stringFor(job.getDataExportTask().getId()), I(job.getDataExportTask().getUserId()), I(job.getDataExportTask().getContextId()), execution);
+
+                        // Store execution in map. Then open it for being processed while registering a clean-up task that ensures execution is removed from map when finished
+                        executions.put(future, execution);
+                        execution.allowProcessing(new AllowProcessingRunnable(future, executions));
+                        openedForProcessing = true;
+                    } finally {
+                        if (!openedForProcessing) {
+                            if (execution != null) {
+                                execution.stop("Failed to start execution");
+                            }
+
+                            if (future != null) {
+                                executions.remove(future);
+                                future.cancel(true);
+                            }
+                        }
+                    }
+                } while (executions.size() < config.getNumberOfConcurrentTasks());
+            } finally {
+                releaseCleanUpTaskLockSafely(lock, acquisition, databaseService);
             }
-        } finally {
-            try {
-                lock.releaseCleanUpTaskLock(acquisition, databaseService);
-            } catch (Exception e) {
-                LOG.warn("Failed to release clean-up lock for data export files", e);
-            }
+        } catch (OXException e) { // NOSONARLINT
+            LOG.warn(Constants.DROP_MDC_MARKER, "Exception while starting data export tasks", e);
+            throw e;
+        } catch (RuntimeException e) { // NOSONARLINT
+            LOG.warn(Constants.DROP_MDC_MARKER, "Exception while starting data export tasks", e);
+            throw DataExportExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+    }
+
+    private static void releaseCleanUpTaskLockSafely(DataExportLock lock, DataExportLock.LockAcquisition acquisition, DatabaseService databaseService) {
+        try {
+            lock.releaseCleanUpTaskLock(acquisition, databaseService);
+        } catch (Exception e) {
+            LOG.warn("Failed to release clean-up lock for data export files", e);
         }
     }
 
@@ -1076,6 +1133,30 @@ public class DataExportServiceImpl implements DataExportService {
         return considerPermissionDeniedErrors;
     }
 
+    /**
+     * Checks if the task's status is done only if the status of all work items are also done.<br>
+     * Changes the task's status if some work items are still in a not done state while the task's status is set to done.
+     *
+     * @param task The task to check
+     */
+    private static void checkAndRepairStatus(DataExportTask task) {
+        if (task == null) {
+            return;
+        }
+        if (task.getStatus() != DataExportStatus.DONE) {
+            return;
+        }
+        if (task.getWorkItems() == null || task.getWorkItems().isEmpty()) {
+            return;
+        }
+        for (DataExportWorkItem item : task.getWorkItems()) {
+            if (item.getStatus() != DataExportStatus.DONE) {
+                task.setStatus(item.getStatus());
+                break;
+            }
+        }
+    }
+
     synchronized void stopProcessingTasks(boolean cancelStartStopTimerTasks, boolean onShutDown) {
         if (cancelStartStopTimerTasks) {
             cancelTimerTasks(periodicStartTask, false, false);
@@ -1086,11 +1167,16 @@ public class DataExportServiceImpl implements DataExportService {
             }
         }
 
-        if (onShutDown || config.isAllowPausingRunningTasks()) {
-            for (Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry : executions.entrySet()) {
-                executionEntry.getValue().stop();
-                executionEntry.getKey().cancel(true);
+        LOG.info("Stopping to process data export tasks due to {}", onShutDown ? "shutdown" : "end of schedule");
+        try {
+            if (onShutDown || config.isAllowPausingRunningTasks()) {
+                for (Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry : executions.entrySet()) {
+                    executionEntry.getValue().stop("Processing window (schedule) has passed");
+                    executionEntry.getKey().cancel(true);
+                }
             }
+        } catch (Exception e) {
+            LOG.warn("Exception while stopping data export tasks", e);
         }
     }
 
@@ -1098,7 +1184,7 @@ public class DataExportServiceImpl implements DataExportService {
 
     private static boolean equals(DataExportTask task1, DataExportTask task2) {
         if (task1 == null) {
-            return task2 == null ? true : false;
+            return task2 == null;
         }
         if (task2 == null) {
             return false;
@@ -1109,7 +1195,7 @@ public class DataExportServiceImpl implements DataExportService {
         if (task1.getUserId() != task2.getUserId()) {
             return false;
         }
-        if (!task1.getId().equals(task2.getId())) {
+        if (!task1.getId().equals(task2.getId())) { // NOSONARLINT
             return false;
         }
         return true;
