@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -46,13 +46,16 @@ import java.util.concurrent.locks.ReentrantLock;
 import org.apache.commons.lang3.tuple.Pair;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.context.ContextService;
-import com.openexchange.context.PoolAndSchema;
 import com.openexchange.database.DatabaseService;
 import com.openexchange.database.Databases;
+import com.openexchange.database.cleanup.CleanUpExecution;
+import com.openexchange.database.cleanup.CleanUpExecutionConnectionProvider;
 import com.openexchange.exception.ExceptionUtils;
 import com.openexchange.exception.OXException;
 import com.openexchange.filestore.FileStorage;
+import com.openexchange.filestore.FileStorageFilter;
 import com.openexchange.filestore.FileStorageInfoService;
+import com.openexchange.filestore.FileStorageService;
 import com.openexchange.filestore.FileStorages;
 import com.openexchange.gdpr.dataexport.DataExportExceptionCode;
 import com.openexchange.gdpr.dataexport.DataExportResultFile;
@@ -73,10 +76,16 @@ import com.openexchange.server.ServiceLookup;
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since v7.10.5
  */
-public class DataExportCleanUpTask implements Runnable {
+public class DataExportCleanUpTask implements CleanUpExecution {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(DataExportCleanUpTask.class);
 
+    private static final String PROP_CLEANUP_ENABLED = "com.openexchange.gdpr.dataexport.cleanup.enabled";
+    private static final String PROP_DATABASE_SERVICE = "com.openexchange.gdpr.dataexport.cleanup.databaseService";
+    private static final String PROP_LOCK_ACQUISITION = "com.openexchange.gdpr.dataexport.cleanup.lockAcquisition";
+    private static final String PROP_CLEANUP_PROCESSED_FILESTORE_IDS = "com.openexchange.gdpr.dataexport.cleanup.processedFilestoreIds";
+    private static final String PROP_CLEANUP_GENERIC_FILESTORE_IDS = "com.openexchange.gdpr.dataexport.cleanup.genericFilestoreIds";
+
     /** The decimal format to use when printing milliseconds */
     private static final NumberFormat MILLIS_FORMAT = newNumberFormat();
 
@@ -97,7 +106,13 @@ public class DataExportCleanUpTask implements Runnable {
         return f;
     }
 
-    private static String formatDuration(long duration) {
+    /**
+     * Formats given duration.
+     *
+     * @param duration The duration
+     * @return The formatted duration
+     */
+    public static String formatDuration(long duration) {
         if (MILLIS_FORMAT_LOCK.tryLock()) {
             try {
                 return MILLIS_FORMAT.format(duration);
@@ -116,6 +131,7 @@ public class DataExportCleanUpTask implements Runnable {
     private final DataExportService dataExportService;
     private final DataExportStorageService storageService;
     private final ServiceLookup services;
+    private final FileStorageFilter fileStorageFilter;
 
     /**
      * Initializes a new {@link DataExportCleanUpTask}.
@@ -129,79 +145,156 @@ public class DataExportCleanUpTask implements Runnable {
         this.dataExportService = dataExportService;
         this.storageService = storageService;
         this.services = services;
-    }
+        this.fileStorageFilter = new FileStorageFilter() {
 
-    private static final String PROP_CLEANUP_ENABLED = "com.openexchange.gdpr.dataexport.cleanup.enabled";
+            @Override
+            public boolean considerUserFileStorages() {
+                return false;
+            }
 
-    private boolean isCleanUpEnabled() {
-        boolean defaultValue = true;
-        ConfigurationService configService = services.getOptionalService(ConfigurationService.class);
-        return configService == null ? defaultValue : configService.getBoolProperty(PROP_CLEANUP_ENABLED, defaultValue);
+            @Override
+            public boolean considerCustomFileStorages() {
+                return true;
+            }
+
+            @Override
+            public boolean considerContextFileStorages() {
+                return false;
+            }
+
+            @Override
+            public boolean acceptUserFileStorages(URI uri) {
+                return false;
+            }
+
+            @Override
+            public boolean acceptCustomFileStorages(URI uri) {
+                return uri.getPath().indexOf("gdpr_dataexport") >= 0;
+            }
+
+            @Override
+            public boolean acceptContextFileStorages(URI uri) {
+                return false;
+            }
+        };
     }
 
     @Override
-    public void run() {
-        if (isCleanUpEnabled() == false) {
-            // Disabled per configuration
-            LOG.info("Aborting clean-up of data export tasks since disabled through configuration");
-            return;
+    public boolean prepareCleanUp(Map<String, Object> state) throws OXException {
+        {
+            boolean defaultValue = true;
+            ConfigurationService configService = services.getOptionalService(ConfigurationService.class);
+            boolean enabled = configService == null ? defaultValue : configService.getBoolProperty(PROP_CLEANUP_ENABLED, defaultValue);
+            if (enabled == false) {
+                LOG.warn("Clean-up of data export files disabled per configuration. Aborting clean-up run");
+                return false;
+            }
         }
 
         DatabaseService databaseService = services.getOptionalService(DatabaseService.class);
         if (databaseService == null) {
             // Missing database service
-            LOG.warn("Failed to acquire database service, which is needed to clean-up data export files. Aborting clean-up run...");
-            return;
+            LOG.warn("Failed to acquire database service, which is needed to clean-up data export files. Aborting clean-up run");
+            return false;
         }
 
+        // Check for possibly currently running data export tasks
         Boolean hasRunningTasks = checkForRunningDataExportTasks();
         if (hasRunningTasks == null) {
             // Check failed and has already been logged
-            return;
+            return false;
         } else if (hasRunningTasks.booleanValue()) {
-            LOG.info("Detected currently running data export tasks. Aborting clean-up run...");
-            return;
+            LOG.info("Detected currently running data export tasks. Aborting clean-up run");
+            return false;
         }
 
+        // Try to acquire lock
+        boolean result = false;
         DataExportLock lock = DataExportLock.getInstance();
-        Thread currentThread = Thread.currentThread();
-        String prevName = currentThread.getName();
-        currentThread.setName("DataExportCleanUpTask");
         DataExportLock.LockAcquisition acquisition = null;
         try {
             acquisition = lock.acquireCleanUpTaskLock(true, databaseService);
             if (acquisition.isNotAcquired()) {
                 // Failed to acquire clean-up lock
-                LOG.info("Failed to acquire clean-up lock for data export files since another process is currently running. Aborting clean-up run...");
-                return;
+                LOG.info("Failed to acquire clean-up lock for data export files since another process is currently running. Aborting clean-up run");
+                return false;
             }
 
-            long start = System.currentTimeMillis();
+            // Lock acquired
+            state.put(PROP_DATABASE_SERVICE, databaseService);
+            state.put(PROP_LOCK_ACQUISITION, acquisition);
+            result = true;
+            return result;
+        } finally {
+            if (result == false) {
+                releaseCleanUpTaskLockSafe(databaseService, acquisition);
+            }
+        }
+    }
 
-            Set<Integer> fileStoreIds = determineFileStorageIdsFromAllScopes(databaseService);
-            if (fileStoreIds == null || fileStoreIds.isEmpty() != false) {
-                LOG.info("Found no file storage(s) for data export files. Aborting clean-up run...");
+    @Override
+    public void finishCleanUp(Map<String, Object> state) throws OXException {
+        DatabaseService databaseService = (DatabaseService) state.get(PROP_DATABASE_SERVICE);
+        DataExportLock.LockAcquisition acquisition = (DataExportLock.LockAcquisition) state.get(PROP_LOCK_ACQUISITION);
+        releaseCleanUpTaskLockSafe(databaseService, acquisition);
+    }
+
+    private static void releaseCleanUpTaskLockSafe(DatabaseService databaseService, DataExportLock.LockAcquisition acquisition) {
+        try {
+            DataExportLock.getInstance().releaseCleanUpTaskLock(acquisition, databaseService);
+        } catch (Exception e) {
+            LOG.warn("Failed to release clean-up lock for data export files", e);
+        }
+    }
+
+    @Override
+    public boolean isApplicableFor(String schema, int representativeContextId, int databasePoolId, Map<String, Object> state, CleanUpExecutionConnectionProvider connectionProvider) throws OXException {
+        return true;
+    }
+
+    @SuppressWarnings("unchecked")
+    @Override
+    public void executeFor(String schema, int representativeContextId, int databasePoolId, Map<String, Object> state, CleanUpExecutionConnectionProvider connectionProvider) {
+        // Determine the file storage identifiers to process for this execution's invocation
+        DatabaseService databaseService = (DatabaseService) state.get(PROP_DATABASE_SERVICE);
+        Set<Integer> fileStoreIdsToProcess;
+        try {
+            Set<Integer> fileStoreIdsWoSchema = (Set<Integer>) state.get(PROP_CLEANUP_GENERIC_FILESTORE_IDS);
+            if (fileStoreIdsWoSchema == null) {
+                fileStoreIdsWoSchema = determineFileStorageIdsFromAllScopesButSchema(databaseService);
+                state.put(PROP_CLEANUP_GENERIC_FILESTORE_IDS, fileStoreIdsWoSchema);
+            }
+            fileStoreIdsToProcess = new HashSet<>(fileStoreIdsWoSchema);
+            fileStoreIdsToProcess.addAll(determineFileStorageIdsFromSchema(databaseService, schema, representativeContextId));
+            Set<Integer> processedFileStoreIds = (Set<Integer>) state.get(PROP_CLEANUP_PROCESSED_FILESTORE_IDS);
+            if (processedFileStoreIds != null) {
+                fileStoreIdsToProcess.removeAll(processedFileStoreIds);
+            }
+            if (fileStoreIdsToProcess.isEmpty()) {
+                LOG.info("Found no file storage(s) to process for data export files. Aborting clean-up run for schema {}", schema);
                 return;
             }
-
-            LOG.info("Going to check {} file storage(s) for orphaned data export files", I(fileStoreIds.size()));
-            fixOrphanedEntries(fileStoreIds);
-
-            long duration = System.currentTimeMillis() - start;
-            LOG.info("Data export clean-up task took {}ms ({})", formatDuration(duration), exactly(duration, true));
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
-            LOG.warn("Failed clean-up run for data exports", t);
-        } finally {
-            if (acquisition != null) {
-                try {
-                    lock.releaseCleanUpTaskLock(acquisition, databaseService);
-                } catch (Exception e) {
-                    LOG.warn("Failed to release clean-up lock for data export files", e);
-                }
-            }
-            currentThread.setName(prevName);
+            LOG.warn("Unable to determine the file storage identifiers to process. Aborting clean-up run for schema {}", schema, t);
+            return;
         }
+
+        // Process determined file storage identifiers
+        try {
+            LOG.info("Going to check {} file storage(s) for orphaned data export files", I(fileStoreIdsToProcess.size()));
+            long start = System.currentTimeMillis();
+            fixOrphanedEntries(fileStoreIdsToProcess);
+            long duration = System.currentTimeMillis() - start;
+            LOG.info("Data export clean-up task took {}ms ({}) for {} file storage(s)", formatDuration(duration), exactly(duration, true), I(fileStoreIdsToProcess.size()));
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            LOG.warn("Failed clean-up run for schema {}", schema, t);
+            return;
+        }
+
+        // Remember already processed file storage identifiers
+        state.put(PROP_CLEANUP_PROCESSED_FILESTORE_IDS, fileStoreIdsToProcess);
     }
 
     private Boolean checkForRunningDataExportTasks() {
@@ -265,13 +358,31 @@ public class DataExportCleanUpTask implements Runnable {
                     addFilesFrom(fileStorage, allFiles);
                 }
             } else {
-                if (allContextIds == null) {
-                    allContextIds = services.getServiceSafe(ContextService.class).getAllContextIds();
+                boolean processed = false;
+                FileStorageService fileStorageService = FileStorages.getFileStorageService();
+                if (fileStorageService != null) {
+                    Optional<List<URI>> optionalDataExportFileStorages = fileStorageService.getFileStorages(baseUri, Optional.of(fileStorageFilter));
+                    if (optionalDataExportFileStorages.isPresent()) {
+                        List<URI> dataExportFileStorages = optionalDataExportFileStorages.get();
+                        for (URI uri : dataExportFileStorages) {
+                            FileStorage fileStorage = fileStorageService.getFileStorage(uri);
+                            if (alreadyVisited.add(fileStorage.getUri())) {
+                                addFilesFrom(fileStorage, allFiles);
+                            }
+                        }
+                        processed = true;
+                    }
                 }
-                for (Integer contextId : allContextIds) {
-                    FileStorage fileStorage = DataExportUtility.getFileStorageFor(baseUri, scheme, i(contextId));
-                    if (alreadyVisited.add(fileStorage.getUri())) {
-                        addFilesFrom(fileStorage, allFiles);
+
+                if (processed == false) {
+                    if (allContextIds == null) {
+                        allContextIds = services.getServiceSafe(ContextService.class).getAllContextIds();
+                    }
+                    for (Integer contextId : allContextIds) {
+                        FileStorage fileStorage = DataExportUtility.getFileStorageFor(baseUri, scheme, i(contextId));
+                        if (alreadyVisited.add(fileStorage.getUri())) {
+                            addFilesFrom(fileStorage, allFiles);
+                        }
                     }
                 }
             }
@@ -347,7 +458,7 @@ public class DataExportCleanUpTask implements Runnable {
 
     // ----------------------------------- Retrieval of file storage identifiers -----------------------------------------------------------
 
-    private Set<Integer> determineFileStorageIdsFromAllScopes(DatabaseService databaseService) {
+    private Set<Integer> determineFileStorageIdsFromAllScopesButSchema(DatabaseService databaseService) {
         Set<Integer> fileStoreIds = null;
 
         // Check for possible configured file storage identifiers for context-sets scope
@@ -393,43 +504,21 @@ public class DataExportCleanUpTask implements Runnable {
         } catch (Exception e) {
             LOG.warn("Failed to retrieve possible identifiers of data export file storage from server scope", e);
         }
-
-        // Check for possible configured file storage identifiers for context and user scope
-        ContextService contextService = services.getOptionalService(ContextService.class);
-        if (contextService != null) {
-            try {
-                for (Integer representativeContextId : contextService.getDistinctContextsPerSchema()) {
-                    fileStoreIds = addFileStoreIdsForSchema(representativeContextId.intValue(), fileStoreIds, databaseService);
-                }
-            } catch (Exception e) {
-                LOG.warn("Failed to retrieve possible identifiers of data export file storage from context and user scope", e);
-            }
-        }
-
         return fileStoreIds == null ? Collections.emptySet() : fileStoreIds;
     }
 
-    private Set<Integer> addFileStoreIdsForSchema(int representativeContextId, Set<Integer> fileStoreIds, DatabaseService databaseService) {
-        Set<Integer> fids = fileStoreIds;
-
+    private Set<Integer> determineFileStorageIdsFromSchema(DatabaseService databaseService, String schema, int representativeContextId) {
+        // Check for possible configured file storage identifiers for schema
+        Set<Integer> fileStoreIds = null;
         try {
             Set<Integer> fileStoreIdsForSchema = determineFileStoreIdsFor(representativeContextId, databaseService);
             if (fileStoreIdsForSchema.isEmpty() == false) {
-                if (fids == null) {
-                    fids = new HashSet<>();
-                }
-                fids.addAll(fileStoreIdsForSchema);
+                fileStoreIds = fileStoreIdsForSchema;
             }
         } catch (Exception e) {
-            Optional<String> optSchema = getSchemaSafe(representativeContextId);
-            if (optSchema.isPresent()) {
-                LOG.warn("Failed to retrieve possible identifiers of data export file storage for schema {}", optSchema.get(), e);
-            } else {
-                LOG.warn("Failed to retrieve possible identifiers of data export file storage for schema association with context {}", I(representativeContextId), e);
-            }
+            LOG.warn("Failed to retrieve possible identifiers of data export file storage for schema {}", schema, e);
         }
-
-        return fids;
+        return fileStoreIds == null ? Collections.emptySet() : fileStoreIds;
     }
 
     private Set<Integer> determineContextSetsFileStoreIds(ConfigurationService configurationService) {
@@ -551,18 +640,4 @@ public class DataExportCleanUpTask implements Runnable {
         }
     }
 
-    private Optional<String> getSchemaSafe(int representativeContextId) {
-        ContextService contextService = services.getOptionalService(ContextService.class);
-        if (contextService == null) {
-            return Optional.empty();
-        }
-
-        try {
-            Map<PoolAndSchema, List<Integer>> associations = contextService.getSchemaAssociationsFor(Collections.singletonList(I(representativeContextId)));
-            return Optional.of(associations.keySet().iterator().next().getSchema());
-        } catch (Exception e) {
-            return Optional.empty();
-        }
-    }
-
 }
