@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@ import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.Set;
 import com.openexchange.database.AbstractCreateTableImpl;
+import com.openexchange.java.Strings;
 
 /**
  * {@link DataExportCreateTableService}
@@ -118,7 +119,7 @@ public class DataExportCreateTableService extends AbstractCreateTableImpl {
 
     @Override
     public String[] requiredTables() {
-        return new String[0];
+        return Strings.getEmptyStrings();
     }
 
     @Override
