@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -33,6 +33,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.groupware.update.PerformParameters;
 import com.openexchange.groupware.update.UpdateExceptionCodes;
 import com.openexchange.groupware.update.UpdateTaskAdapter;
+import com.openexchange.java.Strings;
 
 /**
  * {@link DataExportCreateTableTask}
@@ -44,7 +45,7 @@ public class DataExportCreateTableTask extends UpdateTaskAdapter {
 
     @Override
     public String[] getDependencies() {
-        return new String[0];
+        return Strings.getEmptyStrings();
     }
 
     @Override
@@ -68,7 +69,7 @@ public class DataExportCreateTableTask extends UpdateTaskAdapter {
                 }
                 Databases.autocommit(connection);
             }
-        } 
+        }
     }
 
     private static void createTable(Connection connection, String tableName, String createStatement) throws SQLException {
