@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -165,6 +165,16 @@ public class DataExportNotificationMail {
                 }
                 subject = translator.translate(DataExportNotificationStrings.SUBJECT_FAILURE);
                 break;
+            case EMPTY:
+                content = translator.translate(DataExportNotificationStrings.CONTENT_EMPTY);
+                // Inject creation date
+                {
+                    DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.MEDIUM, user.getLocale());
+                    Date localCreationDate = new Date(creationDate.getTime() + TimeZoneUtils.getTimeZone(user.getTimeZone()).getOffset(creationDate.getTime()));
+                    content = String.format(content, dateFormat.format(localCreationDate));
+                }
+                subject = translator.translate(DataExportNotificationStrings.SUBJECT_EMPTY);
+                break;
             case ABORTED:
                 content = translator.translate(DataExportNotificationStrings.CONTENT_ABORTED);
                 // Inject creation date
