@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -115,12 +115,12 @@ public class DataExportNotificationSender {
                 MailData mailData = DataExportNotificationMail.createNotificationMail(reason, creationDate, expiryDate, hostInfo, userId, contextId);
                 ComposedMailMessage message = notify.createMail(mailData);
                 transport.sendMailMessage(message, ComposeType.NEW);
-                LoggerHolder.LOG.info("Sent {} notification message for data export task {} of user {} in context {}", reason.toString(), stringFor(taskId), I(userId), I(contextId));
+                LoggerHolder.LOG.info("Sent {} notification message for data export task {} of user {} in context {}", reason, stringFor(taskId), I(userId), I(contextId));
             } finally {
                 transport.close();
             }
         } catch (Exception e) {
-            LoggerHolder.LOG.warn("Failed to send {} notification message for data export task {} of user {} in context {}", reason.toString(), stringFor(taskId), I(userId), I(contextId), e);
+            LoggerHolder.LOG.warn("Failed to send {} notification message for data export task {} of user {} in context {}", reason, stringFor(taskId), I(userId), I(contextId), e);
         }
     }
 
