@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -48,6 +48,9 @@ public class DataExportNotificationStrings implements LocalizableStrings {
     // The content of the E-Mail informing about failed data export. E.g. "Unfortunately the data export that you have requested on 09/01/2019 has failed. Please try again."
     public static final String CONTENT_FAILURE = "Unfortunately the data export that you have requested on %1$s has failed. Please try again.";
 
+    // The content of the E-Mail informing about empty data export. E.g. "Unfortunately the data export that you have requested on %1$s could not be generated. No content to export contained in any of selected module(s)."
+    public static final String CONTENT_EMPTY = "Unfortunately the data export that you have requested on %1$s could not be generated. No content to export contained in any of selected module(s).";
+
     // The content of the E-Mail informing about aborted data export. E.g. "The data export that you have requested on 09/01/2019 has been canceled."
     public static final String CONTENT_ABORTED = "The data export that you have requested on %1$s has been canceled.";
 
@@ -57,6 +60,9 @@ public class DataExportNotificationStrings implements LocalizableStrings {
     // The subject of the E-Mail for a failed data export result
     public static final String SUBJECT_FAILURE = "Your personal data export failed";
 
+    // The subject of the E-Mail for a failed data export result
+    public static final String SUBJECT_EMPTY = "Your personal data export failed due to missing content";
+
     // The subject of the E-Mail for an aborted data export result
     public static final String SUBJECT_ABORTED = "Your personal data export has been canceled";
 
