@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -36,6 +36,10 @@ public enum Reason {
      * Data export failed.
      */
     FAILED,
+    /**
+     * No data exported at all since every selected module was empty.
+     */
+    EMPTY,
     /**
      * Data export succeeded.
      */
