@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -38,6 +38,9 @@ import java.util.SortedSet;
 import java.util.UUID;
 import org.apache.commons.lang3.tuple.Pair;
 import com.google.common.collect.ImmutableMap;
+import com.openexchange.auth.AbstractAuthenticatorRMIService;
+import com.openexchange.auth.Authenticator;
+import com.openexchange.auth.Credentials;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
 import com.openexchange.filestore.FileStorage;
@@ -59,7 +62,7 @@ import com.openexchange.java.util.UUIDs;
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since v7.10.3
  */
-public class DataExportRMIServiceImpl implements DataExportRMIService {
+public class DataExportRMIServiceImpl extends AbstractAuthenticatorRMIService implements DataExportRMIService {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(DataExportRMIServiceImpl.class);
 
@@ -75,12 +78,14 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
     public DataExportRMIServiceImpl(DataExportService service) throws OXException {
         super();
         this.service = service;
-        contexts = Services.requireService(ContextService.class);
+        this.contexts = Services.requireService(ContextService.class);
     }
 
     @Override
-    public boolean cancelDataExportTask(int userId, int contextId) throws RemoteException {
+    public boolean cancelDataExportTask(int userId, int contextId, Credentials auth) throws RemoteException {
         try {
+            Authenticator authenticator = Services.requireService(Authenticator.class);
+            authenticateCascaded(authenticator, auth, contextId);
             boolean canceled = service.cancelDataExportTask(userId, contextId);
             if (!canceled) {
                 throw DataExportExceptionCode.CANCEL_TASK_FAILED.create(I(userId), I(contextId));
@@ -96,8 +101,10 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
     }
 
     @Override
-    public List<String> cancelDataExportTasks(int contextId) throws RemoteException {
+    public List<String> cancelDataExportTasks(int contextId, Credentials auth) throws RemoteException {
         try {
+            Authenticator authenticator = Services.optService(Authenticator.class);
+            authenticateCascaded(authenticator, auth, contextId);
             List<UUID> taskIds = service.cancelDataExportTasks(contextId);
             if (taskIds.isEmpty()) {
                 return Collections.emptyList();
@@ -208,7 +215,10 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
     }
 
     @Override
-    public int fixOrphanedEntries(List<Integer> filestoreIds) throws RemoteException {
+    public int fixOrphanedEntries(List<Integer> filestoreIds, Credentials auth) throws RemoteException {
+        Authenticator authenticator = Services.optService(Authenticator.class);
+        authenticate(authenticator, auth);
+
         int fixed = 0;
         try {
             DataExportStorageService storageService = Services.requireService(DataExportStorageService.class);
