@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.gdpr.dataexport.impl.storage;
 
+import static com.openexchange.gdpr.dataexport.impl.DataExportUtility.stringFor;
 import static com.openexchange.java.Autoboxing.B;
 import static com.openexchange.java.Autoboxing.I;
 import java.sql.Connection;
@@ -70,7 +71,11 @@ import com.openexchange.gdpr.dataexport.Module;
 import com.openexchange.gdpr.dataexport.impl.DataExportUtility;
 import com.openexchange.gdpr.dataexport.impl.osgi.Services;
 import com.openexchange.gdpr.dataexport.impl.utils.FileStorageAndId;
+import com.openexchange.java.MessageFormatter;
+import com.openexchange.java.util.ImmutablePair;
 import com.openexchange.java.util.UUIDs;
+import com.openexchange.server.ServiceLookup;
+import com.openexchange.user.UserService;
 
 /**
  * {@link AbstractDataExportSql} - The abstract SQL access to data export storage.
@@ -99,6 +104,9 @@ public abstract class AbstractDataExportSql<R> {
     /** The database service */
     protected final DatabaseService databaseService;
 
+    /** The service look-up */
+    protected final ServiceLookup services;
+
     private final DataExportConfig config;
     private final ConcurrentMap<String, Boolean> tableExistsCache;
 
@@ -107,11 +115,13 @@ public abstract class AbstractDataExportSql<R> {
      *
      * @param databaseService The database service
      * @param config The configuration
+     * @param services The service look-up
      */
-    protected AbstractDataExportSql(DatabaseService databaseService, DataExportConfig config) {
+    protected AbstractDataExportSql(DatabaseService databaseService, DataExportConfig config, ServiceLookup services) {
         super();
         this.databaseService = databaseService;
         this.config = config;
+        this.services = services;
         tableExistsCache = new ConcurrentHashMap<String, Boolean>();
     }
 
@@ -370,7 +380,7 @@ public abstract class AbstractDataExportSql<R> {
         long expirationThreshold = now - config.getExpirationTimeMillis();
         long maxTimeToLiveThreshold = now - config.getMaxTimeToLiveMillis();
 
-        List<TaskInfo> tasksToDelete = null;
+        List<ImmutablePair<TaskInfo, R>> tasksToDelete = null;
         List<DataExportTaskInfo> taskInfos = null;
         for (R schemaReference : getSchemaReferences()) {
             Connection connection = null;
@@ -390,7 +400,9 @@ public abstract class AbstractDataExportSql<R> {
                                     if (tasksToDelete == null) {
                                         tasksToDelete = new ArrayList<>();
                                     }
-                                    tasksToDelete.add(new TaskInfo(UUIDs.toUUID(rs.getBytes(2)/* uuid */), 0, rs.getInt(4)/* user */, rs.getInt(1)/* cid */));
+                                    TaskInfo taskInfo = new TaskInfo(UUIDs.toUUID(rs.getBytes(2)/* uuid */), 0, rs.getInt(4)/* user */, rs.getInt(1)/* cid */);
+                                    tasksToDelete.add(ImmutablePair.<TaskInfo, R>newInstance(taskInfo, schemaReference));
+                                    LOGGER.debug("Deleting {} from storage since {}", taskInfo, status.getId());
                                 } else if (status.isDone() || status.isFailed()) {
                                     UUID taskId = UUIDs.toUUID(rs.getBytes(2)/* uuid */);
                                     // Check if part of result set because expired or has pending notification
@@ -398,12 +410,32 @@ public abstract class AbstractDataExportSql<R> {
                                         if (tasksToDelete == null) {
                                             tasksToDelete = new ArrayList<>();
                                         }
-                                        tasksToDelete.add(new TaskInfo(taskId, 0, rs.getInt(4)/* user */, rs.getInt(1)/* cid */));
+                                        TaskInfo taskInfo = new TaskInfo(taskId, 0, rs.getInt(4)/* user */, rs.getInt(1)/* cid */);
+                                        tasksToDelete.add(ImmutablePair.<TaskInfo, R>newInstance(taskInfo, schemaReference));
+                                        LOGGER.debug("Deleting {} from storage since {}", taskInfo, status.getId());
                                     } else {
-                                        if (taskInfos == null) {
-                                            taskInfos = new ArrayList<>();
+                                        // Check user existence
+                                        int contextId = rs.getInt(1); /* cid */
+                                        int userId = rs.getInt(4); /* user */
+                                        boolean exists;
+                                        {
+                                            UserService userService = services.getOptionalService(UserService.class);
+                                            exists = userService == null || userService.exists(userId, contextId);
+                                        }
+                                        // Add to either collection dependent on existence
+                                        if (exists) {
+                                            if (taskInfos == null) {
+                                                taskInfos = new ArrayList<>();
+                                            }
+                                            taskInfos.add(new DataExportTaskInfo(taskId, userId, contextId, status));
+                                        } else {
+                                            if (tasksToDelete == null) {
+                                                tasksToDelete = new ArrayList<>();
+                                            }
+                                            TaskInfo taskInfo = new TaskInfo(taskId, 0, userId, contextId);
+                                            tasksToDelete.add(ImmutablePair.<TaskInfo, R>newInstance(taskInfo, schemaReference));
+                                            LOGGER.debug("Deleting {} from storage since user no more exists", taskInfo);
                                         }
-                                        taskInfos.add(new DataExportTaskInfo(taskId, rs.getInt(4)/* user */, rs.getInt(1)/* cid */, status));
                                     }
                                 }
                             }
@@ -420,8 +452,8 @@ public abstract class AbstractDataExportSql<R> {
         }
 
         if (tasksToDelete != null) {
-            for (TaskInfo taskInfo : tasksToDelete) {
-                deleteTask(taskInfo.taskId, taskInfo.userId,  taskInfo.contextId);
+            for (ImmutablePair<TaskInfo, R> taskInfoAndReference : tasksToDelete) {
+                deleteTask(taskInfoAndReference.getFirst().taskId, taskInfoAndReference.getSecond());
             }
         }
 
@@ -1430,12 +1462,15 @@ public abstract class AbstractDataExportSql<R> {
         R schemaReference = getSchemaReference(userId, contextId);
         boolean modified = false;
         Connection con = getWritable(schemaReference);
-        try (PreparedStatement stmt = con.prepareStatement("INSERT INTO dataExportFilestoreLocation (cid, taskId, num, filestoreLocation, size) VALUES (?, ?, ?, ?, ?)")) {
+        try (PreparedStatement stmt = con.prepareStatement("INSERT INTO dataExportFilestoreLocation (cid, taskId, num, filestoreLocation, size) VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `cid` = ?, `filestoreLocation` = ?, `size` = ?")) {
             stmt.setInt(1, contextId);
             stmt.setBytes(2, UUIDs.toByteArray(taskId));
             stmt.setInt(3, number);
             stmt.setString(4, fileStorageLocation);
             stmt.setLong(5, size);
+            stmt.setInt(6, contextId);
+            stmt.setString(7, fileStorageLocation);
+            stmt.setLong(8, size);
             modified = stmt.executeUpdate() > 0;
         } catch (SQLException e) {
             throw handleException(e);
@@ -2053,6 +2088,11 @@ public abstract class AbstractDataExportSql<R> {
             long ostamp = o.timestamp;
             return (timestamp < ostamp) ? -1 : ((timestamp == ostamp) ? 0 : 1);
         }
+
+        @Override
+        public String toString() {
+            return MessageFormatter.format("data export task {} of user {} in context {}", stringFor(taskId), I(userId), I(contextId));
+        }
     }
 
 }
