@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -48,6 +48,7 @@ import com.openexchange.chronos.common.mapping.EventMapper;
 import com.openexchange.chronos.ical.CalendarExport;
 import com.openexchange.chronos.ical.ICalParameters;
 import com.openexchange.chronos.ical.ICalService;
+import com.openexchange.chronos.provider.composition.IDMangling;
 import com.openexchange.chronos.service.CalendarParameters;
 import com.openexchange.chronos.service.CalendarService;
 import com.openexchange.chronos.service.CalendarSession;
@@ -263,7 +264,7 @@ public class CalendarDataExport extends AbstractDataExportProviderTask {
             List<Folder> children;
             try {
                 FolderService folderService = neededServices.folderService;
-                UserizedFolder[] visibleFolders = folderService.getVisibleFolders(folder.getFolderId(), FolderStorage.REAL_TREE_ID, decoratorProvider.contentType, type, options.includeUnsubscribed, session, decoratorProvider.createDecorator()).getResponse();
+                UserizedFolder[] visibleFolders = folderService.getVisibleFolders(FolderStorage.REAL_TREE_ID, decoratorProvider.contentType, type, options.includeUnsubscribed, session, decoratorProvider.createDecorator()).getResponse();
                 if (null == visibleFolders || visibleFolders.length <= 0) {
                     children = Collections.emptyList();
                 } else {
@@ -274,6 +275,10 @@ public class CalendarDataExport extends AbstractDataExportProviderTask {
 
                     children = new ArrayList<>(visibleFolders.length);
                     for (UserizedFolder subfolder : visibleFolders) {
+                        if (IDMangling.getRelativeFolderId(subfolder.getID()).equals(folder.getFolderId())) {
+                            // Skip parent folder to prevent endless recursion
+                            continue;
+                        }
                         if (subfolder.getID().startsWith(CalendarUtils.DEFAULT_ACCOUNT_PREFIX)) {
                             String folderId = subfolder.getID().substring(CalendarUtils.DEFAULT_ACCOUNT_PREFIX.length());
                             String namePrefix = null;
@@ -281,7 +286,7 @@ public class CalendarDataExport extends AbstractDataExportProviderTask {
                                 FullNameBuilderService fullNameBuilder = services.getServiceSafe(FullNameBuilderService.class);
                                 namePrefix = generateFullNamePrefix(fullNameBuilder.buildFullName(subfolder.getCreatedBy(), task.getContextId(), translator));
                             }
-                            children.add(new Folder(folderId, namePrefix == null ? subfolder.getLocalizedName(locale) : namePrefix + subfolder.getLocalizedName(locale), false));
+                            children.add(new Folder(folderId, namePrefix == null ? subfolder.getName(locale) : namePrefix + subfolder.getName(locale), false));
                         }
                     }
                 }
