@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -54,6 +54,7 @@ import com.openexchange.folderstorage.FolderResponse;
 import com.openexchange.folderstorage.FolderService;
 import com.openexchange.folderstorage.FolderServiceDecorator;
 import com.openexchange.folderstorage.FolderStorage;
+import com.openexchange.folderstorage.GlobalAddressBookUtils;
 import com.openexchange.folderstorage.Permission;
 import com.openexchange.folderstorage.Type;
 import com.openexchange.folderstorage.UserizedFolder;
@@ -195,7 +196,7 @@ public class ContactsDataExport extends AbstractDataExportProviderTask {
                     Permission permission = CalculatePermission.calculate(globalAddressBook, user, context, Collections.singletonList(contentType));
                     if (permission.isVisible() && permission.getReadPermission() >= Permission.READ_ALL_OBJECTS) {
                         exportSingleUserContact = false;
-                        String name = helper.getString(FolderStrings.SYSTEM_LDAP_FOLDER_NAME);
+                        String name = GlobalAddressBookUtils.getFolderName(locale, context.getContextId());
                         Folder folder = new Folder(Integer.toString(FolderObject.SYSTEM_LDAP_FOLDER_ID), name, false);
                         String pathOfFolder = exportFolder(folder, null);
                         if (pathOfFolder == null) {
@@ -212,7 +213,7 @@ public class ContactsDataExport extends AbstractDataExportProviderTask {
                 if (exportSingleUserContact) {
                     Contact userContact = contactService.getUser(session, session.getUserId());
 
-                    String name = helper.getString(FolderStrings.SYSTEM_LDAP_FOLDER_NAME);
+                    String name = GlobalAddressBookUtils.getFolderName(locale, context.getContextId());
                     Folder folder = new Folder(Integer.toString(FolderObject.SYSTEM_LDAP_FOLDER_ID), name, false);
                     String pathOfFolder = exportFolder(folder, null);
                     if (pathOfFolder == null) {
@@ -336,7 +337,7 @@ public class ContactsDataExport extends AbstractDataExportProviderTask {
             List<Folder> allVisible;
             try {
                 FolderService folderService = neededServices.folderService;
-                FolderResponse<UserizedFolder[]> visibleFoldersResponse = folderService.getVisibleFolders(folder.getFolderId(), FolderStorage.REAL_TREE_ID, decoratorProvider.getContentType(), type, true, session, decoratorProvider.createDecorator());
+                FolderResponse<UserizedFolder[]> visibleFoldersResponse = folderService.getVisibleFolders(FolderStorage.REAL_TREE_ID, decoratorProvider.getContentType(), type, true, session, decoratorProvider.createDecorator());
                 UserizedFolder[] visibleFolders = visibleFoldersResponse.getResponse();
                 if (null == visibleFolders || visibleFolders.length <= 0) {
                     allVisible = Collections.emptyList();
@@ -348,13 +349,17 @@ public class ContactsDataExport extends AbstractDataExportProviderTask {
 
                     allVisible = new ArrayList<>(visibleFolders.length);
                     for (UserizedFolder subfolder : visibleFolders) {
+                        if (subfolder.getID().equals(folder.getFolderId())) {
+                            // Skip parent folder to prevent endless recursion
+                            continue;
+                        }
                         if (!GLOBAL_ADDRESS_BOOK_ID.equals(subfolder.getID())) {
                             String namePrefix = null;
                             if (translator != null) {
                                 FullNameBuilderService fullNameBuilder = services.getServiceSafe(FullNameBuilderService.class);
                                 namePrefix = generateFullNamePrefix(fullNameBuilder.buildFullName(subfolder.getCreatedBy(), task.getContextId(), translator));
                             }
-                            allVisible.add(new Folder(subfolder.getID(), namePrefix == null ? subfolder.getLocalizedName(locale) : namePrefix + subfolder.getLocalizedName(locale), false));
+                            allVisible.add(new Folder(subfolder.getID(), namePrefix == null ? subfolder.getName(locale) : namePrefix + subfolder.getName(locale), false));
                         }
                     }
                 }
