@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -134,7 +134,7 @@ public class AttachmentLoader {
                 }
             }
 
-            fileHolder = new ThresholdFileHolder();
+            fileHolder = new ThresholdFileHolder(false);
             InputStream documentData = ATTACHMENT_BASE.getAttachedFile(session, folderId, contactId, moduleId, attachment.getId(), ctx, user, userConfig);
             try {
                 fileHolder.write(documentData);
