@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -278,7 +278,7 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
                 if (pathOfFolder == null) {
                     return savePointFor(new JSONObject(2).putSafe("folder", folder.getFolderId()).putSafe("root", root));
                 }
-                LOG.debug("Exported infostore directory {} for data export {} of user {} in context {}", folder.getName(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                LOG.debug("Exported infostore directory \"{}\" for data export {} of user {} in context {}", folder.getName(), UUIDs.getUnformattedStringObjectFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
             } else {
                 pathOfFolder = startInfo.path;
             }
@@ -299,13 +299,13 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
                 switch (root) {
                     case PRIVATE:
                         FileStorageFolder personalFolder = infostoreAccountAccess.getFolderAccess().getPersonalFolder();
-                        children = Collections.singletonList(new Folder(personalFolder.getId(), personalFolder.getLocalizedName(locale), false));
+                        children = Collections.singletonList(new Folder(personalFolder.getId(), personalFolder.getName(locale), false));
                         break;
                     case PUBLIC:
                         FileStorageFolder[] publicFolders = infostoreAccountAccess.getFolderAccess().getPublicFolders();
                         children = new ArrayList<>(publicFolders.length);
                         for (FileStorageFolder publicFolder : publicFolders) {
-                            children.add(new Folder(publicFolder.getId(), publicFolder.getLocalizedName(locale), false));
+                            children.add(new Folder(publicFolder.getId(), publicFolder.getName(locale), false));
                         }
                         break;
                     case SHARED:
@@ -334,14 +334,14 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
                                 if (!id.equals(persFolder.getID()) && !id.equals(trashFolder.getID())) {
                                     FullNameBuilderService fullNameBuilder = services.getServiceSafe(FullNameBuilderService.class);
                                     String namePrefix = generateFullNamePrefix(fullNameBuilder.buildFullName(subfolder.getCreatedBy(), task.getContextId(), translator));
-                                    children.add(new Folder(id, namePrefix + subfolder.getLocalizedName(locale), false));
+                                    children.add(new Folder(id, namePrefix + subfolder.getName(locale), false));
                                 }
                             }
                         }
                         break;
                     case TRASH:
                         FileStorageFolder trashFolder =infostoreAccountAccess.getFolderAccess().getTrashFolder();
-                        children = Collections.singletonList(new Folder(trashFolder.getId(), trashFolder.getLocalizedName(locale), false));
+                        children = Collections.singletonList(new Folder(trashFolder.getId(), trashFolder.getName(locale), false));
                         break;
                     default:
                         throw new IllegalArgumentException("Unknwn root: " + root.getId());
@@ -353,7 +353,7 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
                 } else {
                     children = new ArrayList<>(subfolders.length);
                     for (FileStorageFolder subfolder : subfolders) {
-                        children.add(new Folder(subfolder.getId(), subfolder.getLocalizedName(locale), false));
+                        children.add(new Folder(subfolder.getId(), subfolder.getName(locale), false));
                     }
                 }
             }
@@ -573,7 +573,7 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
         for (List<File> versions : fileToVersions.values()) {
             total += versions.size();
         }
-        LOG.debug("Going to export {} file versions from directory {} for data export {} of user {} in context {}", I(total), folder.getName(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+        LOG.debug("Going to export {} file versions from directory \"{}\" for data export {} of user {} in context {}", I(total), folder.getName(), UUIDs.getUnformattedStringObjectFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
 
         int batchCount = 0;
         for (List<File> versions : fileToVersions.values()) {
@@ -607,7 +607,7 @@ public class InfostoreDataExport extends AbstractDataExportProviderTask {
                     if (!exported) {
                         return savePointFor(new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", root).putSafe("id", version.getId()).putSafe("version", version.getVersion()));
                     }
-                    LOG.debug("Exported version {} of file {} ({} of {}) from directory {} for data export {} of user {} in context {}", version.getVersion(), version.getId(), I(batchCount), I(total), folder.getName(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                    LOG.debug("Exported version {} of file {} ({} of {}) from directory \"{}\" for data export {} of user {} in context {}", version.getVersion(), version.getId(), I(batchCount), I(total), folder.getName(), UUIDs.getUnformattedStringObjectFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
                 } catch (Exception e) {
                     if (isRetryableExceptionAndMayFail(e, sink)) {
                         return savePointFor(new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", root).putSafe("id", version.getId()).putSafe("version", version.getVersion()), e);
