@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -106,16 +106,16 @@ public class InfostoreDataExportProvider extends AbstractDataExportProvider<Info
 
         Map<String, Object> properties = new LinkedHashMap<String, Object>(6);
         properties.put(InfostoreDataExportPropertyNames.PROP_ENABLED, Boolean.TRUE);
-        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.calendar." + InfostoreDataExportPropertyNames.PROP_INCLUDE_PUBLIC_FOLDERS, true, session)) {
+        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.infostore." + InfostoreDataExportPropertyNames.PROP_INCLUDE_PUBLIC_FOLDERS, true, session)) {
             properties.put(InfostoreDataExportPropertyNames.PROP_INCLUDE_PUBLIC_FOLDERS, Boolean.FALSE);
         }
-        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.calendar." + InfostoreDataExportPropertyNames.PROP_INCLUDE_SHARED_FOLDERS, true, session)) {
+        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.infostore." + InfostoreDataExportPropertyNames.PROP_INCLUDE_SHARED_FOLDERS, true, session)) {
             properties.put(InfostoreDataExportPropertyNames.PROP_INCLUDE_SHARED_FOLDERS, Boolean.FALSE);
         }
-        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.calendar." + InfostoreDataExportPropertyNames.PROP_INCLUDE_TRASH, true, session)) {
+        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.infostore." + InfostoreDataExportPropertyNames.PROP_INCLUDE_TRASH, true, session)) {
             properties.put(InfostoreDataExportPropertyNames.PROP_INCLUDE_TRASH, Boolean.FALSE);
         }
-        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.calendar." + InfostoreDataExportPropertyNames.PROP_INCLUDE_ALL_VERSIONS, true, session)) {
+        if (getBoolProperty("com.openexchange.gdpr.dataexport.provider.infostore." + InfostoreDataExportPropertyNames.PROP_INCLUDE_ALL_VERSIONS, true, session)) {
             properties.put(InfostoreDataExportPropertyNames.PROP_INCLUDE_ALL_VERSIONS, Boolean.FALSE);
         }
 
