@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 
 package com.openexchange.gdpr.dataexport.provider.mail.generator;
 
-import static com.openexchange.mail.api.MailConfig.getConfiguredAuthType;
+import static com.openexchange.mail.api.MailConfig.getConfiguredAuthTypeForMailAccess;
 import java.util.LinkedHashMap;
 import java.util.Map;
 import com.openexchange.exception.OXException;
@@ -30,6 +30,7 @@ import com.openexchange.gdpr.dataexport.provider.mail.MailDataExportPropertyName
 import com.openexchange.mail.api.AuthType;
 import com.openexchange.mail.api.MailConfig.LoginSource;
 import com.openexchange.mail.config.MailProperties;
+import com.openexchange.mailaccount.AccountNature;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
@@ -66,7 +67,7 @@ public abstract class AbstractSessionGenerator implements SessionGenerator {
 
     @Override
     public boolean isApplicable(Session session) throws OXException {
-        AuthType configuredAuthType = getConfiguredAuthType(true, session);
+        AuthType configuredAuthType = getConfiguredAuthTypeForMailAccess(AccountNature.PRIMARY, session);
         return isApplicable(AuthType.isOAuthType(configuredAuthType), session);
     }
 
