@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -73,12 +73,12 @@ public class DefaultFolder implements Folder {
      * Initializes a new {@link DefaultFolder}.
      *
      * @param folderInfo The folder info
-     * @param infoSupport The folder info access
+     * @param mailOperationExecutor The mail access reference
      */
-    public DefaultFolder(MailFolderInfo folderInfo, IMailFolderStorageInfoSupport infoSupport) {
+    public DefaultFolder(MailFolderInfo folderInfo, MailOperationExecutor mailOperationExecutor) {
         super();
         String fullname = folderInfo.getFullname();
-        folderAccess = new MailFolderInfoAccess(fullname, infoSupport);
+        folderAccess = new MailAccessReferenceAccess(fullname, mailOperationExecutor);
         this.fullname = fullname;
         trash = folderInfo.isTrash();
         name = folderInfo.getName();
@@ -94,12 +94,12 @@ public class DefaultFolder implements Folder {
      * Initializes a new {@link DefaultFolder}.
      *
      * @param folder The folder
-     * @param folderStorage The folder access
+     * @param mailOperationExecutor The mail access reference
      */
-    public DefaultFolder(MailFolder folder, IMailFolderStorage folderStorage) {
+    public DefaultFolder(MailFolder folder, MailOperationExecutor mailOperationExecutor) {
         super();
         String fullname = folder.getFullname();
-        folderAccess = new MailFolderAccess(fullname, folderStorage);
+        folderAccess = new MailAccessReferenceAccess(fullname, mailOperationExecutor);
         this.fullname = fullname;
         trash = folder.isTrash();
         name = folder.getName();
@@ -163,25 +163,30 @@ public class DefaultFolder implements Folder {
 
     // ------------------------------------------------------ Helpers ----------------------------------------------------------------------
 
-    private static class MailFolderInfoAccess implements FolderAccess {
+    private static class MailAccessReferenceAccess implements FolderAccess {
 
-        private final IMailFolderStorageInfoSupport infoSupport;
+        private final MailOperationExecutor mailOperationExecutor;
         private final String fullname;
 
-        /**
-         * Initializes a new {@link MailFolderInfoAccess}.
-         *
-         * @param fullname The folder full name
-         * @param infoSupport The folder info access
-         */
-        MailFolderInfoAccess(String fullname, IMailFolderStorageInfoSupport infoSupport) {
+        MailAccessReferenceAccess(String fullname, MailOperationExecutor mailOperationExecutor) {
             super();
-            this.infoSupport = infoSupport;
+            this.mailOperationExecutor = mailOperationExecutor;
             this.fullname = fullname;
         }
 
         @Override
         public List<Folder> getChildren(boolean subscribedOnly) throws OXException {
+            return mailOperationExecutor.executeWithRetryOnConnectionLoss(mailAccess -> {
+                IMailFolderStorage folderStorage = mailAccess.getFolderStorage();
+                IMailFolderStorageInfoSupport infoSupport = folderStorage.supports(IMailFolderStorageInfoSupport.class);
+                if (null != infoSupport && infoSupport.isInfoSupported()) {
+                    return getChildrenByInfoSupport(subscribedOnly, infoSupport);
+                }
+                return getChildrenByFolderStorage(subscribedOnly, folderStorage);
+            });
+        }
+
+        private List<Folder> getChildrenByInfoSupport(boolean subscribedOnly, IMailFolderStorageInfoSupport infoSupport) throws OXException {
             List<MailFolderInfo> children = infoSupport.getFolderInfos(fullname, subscribedOnly);
             if (children == null) {
                 return Collections.emptyList();
@@ -198,32 +203,13 @@ public class DefaultFolder implements Folder {
             List<Folder> retval = new ArrayList<>(size);
             for (MailFolderInfo mailFolderInfo : children) {
                 if (ignoreFullNamesToIgnore || !fullNamesToIgnore.contains(mailFolderInfo.getFullname())) {
-                    retval.add(new DefaultFolder(mailFolderInfo, infoSupport));
+                    retval.add(new DefaultFolder(mailFolderInfo, mailOperationExecutor));
                 }
             }
             return retval;
         }
-    }
 
-    private static class MailFolderAccess implements FolderAccess {
-
-        private final IMailFolderStorage folderStorage;
-        private final String fullname;
-
-        /**
-         * Initializes a new {@link MailFolderAccess}.
-         *
-         * @param fullname The folder full name
-         * @param folderStorage The folder access
-         */
-        MailFolderAccess(String fullname, IMailFolderStorage folderStorage) {
-            super();
-            this.folderStorage = folderStorage;
-            this.fullname = fullname;
-        }
-
-        @Override
-        public List<Folder> getChildren(boolean subscribedOnly) throws OXException {
+        private List<Folder> getChildrenByFolderStorage(boolean subscribedOnly, IMailFolderStorage folderStorage) throws OXException {
             MailFolder[] children = folderStorage.getSubfolders(fullname, !subscribedOnly);
             if (children == null || children.length <= 0) {
                 return Collections.emptyList();
@@ -235,12 +221,11 @@ public class DefaultFolder implements Folder {
             List<Folder> retval = new ArrayList<>(children.length);
             for (MailFolder mailFolder : children) {
                 if (ignoreFullNamesToIgnore || !fullNamesToIgnore.contains(mailFolder.getFullname())) {
-                    retval.add(new DefaultFolder(mailFolder, folderStorage));
+                    retval.add(new DefaultFolder(mailFolder, mailOperationExecutor));
                 }
             }
             return retval;
         }
-
     }
 
 }
