@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -35,8 +35,6 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Optional;
-import java.util.concurrent.Callable;
-import java.util.concurrent.TimeUnit;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 import org.json.JSONException;
@@ -61,11 +59,9 @@ import com.openexchange.gdpr.dataexport.provider.mail.MailDataExportPropertyName
 import com.openexchange.gdpr.dataexport.provider.mail.generator.FailedAuthenticationResult;
 import com.openexchange.gdpr.dataexport.provider.mail.generator.SessionGenerator;
 import com.openexchange.java.Collators;
-import com.openexchange.java.Reference;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
-import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.MailField;
 import com.openexchange.mail.MailSortField;
 import com.openexchange.mail.OrderDirection;
@@ -75,15 +71,9 @@ import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.MailMessage;
-import com.openexchange.mail.mime.MimeMailExceptionCode;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.service.MailService;
 import com.openexchange.server.ServiceLookup;
-import com.openexchange.session.Session;
-import net.jodah.failsafe.Failsafe;
-import net.jodah.failsafe.FailsafeException;
-import net.jodah.failsafe.RetryPolicy;
-import net.jodah.failsafe.function.CheckedConsumer;
 
 /**
  * {@link MailDataExport}
@@ -94,7 +84,7 @@ import net.jodah.failsafe.function.CheckedConsumer;
 public class MailDataExport extends AbstractDataExportProviderTask {
 
     /** The logger constant */
-    static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailDataExport.class);
+    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailDataExport.class);
 
     private static final String ID_MAIL = "mail";
 
@@ -104,10 +94,6 @@ public class MailDataExport extends AbstractDataExportProviderTask {
         ConfigView view = factory.getView(userId, contextId);
 
         ComposedConfigProperty<String> property = view.property("com.openexchange.find.basic.mail.allMessagesFolder", String.class);
-        if (false == property.isDefined()) {
-            return null;
-        }
-
         String fn = property.get();
         return Strings.isEmpty(fn) ? null : fn;
     }
@@ -141,7 +127,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
     public ExportResult export() throws OXException {
         MailService mailService = services.getServiceSafe(MailService.class);
 
-        Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference = new Reference<>();
+        MailOperationExecutor mailOperationExecutor = null;
         try {
             Module mailModule = getModule();
 
@@ -173,24 +159,30 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             tryTouch();
 
             // Connect
-            boolean tryConnect = true;
-            while (tryConnect) {
+            while (mailOperationExecutor == null) {
+                MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess = null;
                 try {
-                    mailAccessReference.setValue(mailService.getMailAccess(session, 0));
-                    mailAccessReference.getValue().connect();
-                    tryConnect = false;
+                    mailAccess = mailService.getMailAccess(session, 0);
+                    mailAccess.connect();
+                    mailOperationExecutor = new MailOperationExecutor(mailModule, sessionGenerator, mailAccess, task, services);
+                    mailAccess = null;
                 } catch (OXException e) {
                     // Check for failed authentication
-                    if (!com.openexchange.mail.api.MailAccess.isAuthFailed(e)) {
+                    if (!MailAccess.isAuthFailed(e)) {
                         throw e;
                     }
 
                     // Retry with new generated session
+                    if (mailAccess != null) {
+                        LOG.debug("Failed authentication against mail server '{}' during data export {} of user {} in context {}. Trying to establish a new session...", mailAccess.getMailConfig().getServer(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                    }
                     FailedAuthenticationResult failedAuthenticationResult = sessionGenerator.onFailedAuthentication(e, session, mailModule.getProperties().get());
                     if (!failedAuthenticationResult.retry()) {
                         throw e;
                     }
                     session = failedAuthenticationResult.getOptionalSession().get();
+                } finally {
+                    MailAccess.closeInstance(mailAccess);
                 }
             }
 
@@ -199,25 +191,24 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 DefaultFolder.setFullNamesToIgnore(Collections.singleton(allMessagesFolder));
             }
 
-            Folder rootFolder;
-            {
-                @SuppressWarnings("null") IMailFolderStorage folderStorage = mailAccessReference.getValue().getFolderStorage(); // mailAccess is guarded by 'tryConnect'
+            MailOperationExecutor executor = mailOperationExecutor;
+            Folder rootFolder =  mailOperationExecutor.executeWithRetryOnConnectionLoss(mailAccess -> {
+                IMailFolderStorage folderStorage = mailAccess.getFolderStorage();
                 IMailFolderStorageInfoSupport infoSupport = folderStorage.supports(IMailFolderStorageInfoSupport.class);
                 if (null != infoSupport && infoSupport.isInfoSupported()) {
-                    rootFolder = new DefaultFolder(infoSupport.getFolderInfo(MailFolder.ROOT_FOLDER_ID), infoSupport);
-                } else {
-                    rootFolder = new DefaultFolder(folderStorage.getRootFolder(), folderStorage);
+                    return new DefaultFolder(infoSupport.getFolderInfo(MailFolder.ROOT_FOLDER_ID), executor);
                 }
-            }
+                return new DefaultFolder(folderStorage.getRootFolder(), executor);
+            });
 
-            mailAccessReference.getValue().setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
+            mailOperationExecutor.getCurrentMailAccess().setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
             try {
-                SavePointAndReason optSavePoint = traverseFolder(rootFolder, null, startInfo, options, mailAccessReference);
+                SavePointAndReason optSavePoint = traverseFolder(rootFolder, null, startInfo, options, mailOperationExecutor);
                 if (optSavePoint != null) {
                     return optSavePoint.result();
                 }
             } finally {
-                mailAccessReference.getValue().setWaiting(false); // Unset "waiting" flag
+                mailOperationExecutor.getCurrentMailAccess().setWaiting(false); // Unset "waiting" flag
             }
 
             tryTouch();
@@ -237,11 +228,11 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             }
             throw OXException.general(e.getMessage(), e);
         } finally {
-            MailAccess.closeInstance(mailAccessReference.getValue());
+            MailAccess.closeInstance(mailOperationExecutor == null ? null : mailOperationExecutor.getCurrentMailAccess());
         }
     }
 
-    private SavePointAndReason traverseFolder(Folder folder, String parentPath, StartInfo startInfo, Options options, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference) throws OXException, DataExportAbortedException {
+    private SavePointAndReason traverseFolder(Folder folder, String parentPath, StartInfo startInfo, Options options, MailOperationExecutor mailOperationExecutor) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
             if (startInfo != null) {
                 JSONObject jSavePoint = new JSONObject(4);
@@ -267,13 +258,13 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 if (pathOfFolder == null) {
                     return savePointFor(new JSONObject(2).putSafe("folder", folder.getFullname()));
                 }
-                LOG.debug("Exported mail directory {} for data export {} of user {} in context {}", folder.getFullname(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                LOG.debug("Exported mail directory \"{}\" for data export {} of user {} in context {}", folder.getFullname(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
             } else {
                 pathOfFolder = info.path;
             }
 
             if (folder.isHoldsMessages()) {
-                SavePointAndReason jSavePoint = exportMessages(folder.getFullname(), pathOfFolder, info == null ? null : info.mailId, mailAccessReference);
+                SavePointAndReason jSavePoint = exportMessages(folder.getFullname(), pathOfFolder, info == null ? null : info.mailId, mailOperationExecutor);
                 if (jSavePoint != null) {
                     return jSavePoint;
                 }
@@ -301,7 +292,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
                 return savePointFor(new JSONObject(2).putSafe("folder", folder.getFullname()), e);
             }
-            LOG.warn("Failed to retrieve subfolders of folder \"{}\" from primary mail account of user {} in context {}", folder.getFullname(), I(task.getUserId()), I(task.getContextId()), e);
+            LOG.warn("Failed to retrieve subfolders of directory \"{}\" from primary mail account for data export {} of user {} in context {}", folder.getFullname(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
             sink.addToReport(Message.builder().appendToMessage("Failed to retrieve subfolders of folder \"").appendToMessage(folder.getFullname()).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_MAIL).withTimeStamp(new Date()).build());
             children = Collections.emptyList();
         }
@@ -317,7 +308,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 }
             }
             for (Folder child : children) {
-                SavePointAndReason jSavePoint = traverseFolder(child, pathOfFolder, info, options, mailAccessReference);
+                SavePointAndReason jSavePoint = traverseFolder(child, pathOfFolder, info, options, mailOperationExecutor);
                 if (jSavePoint != null) {
                     return jSavePoint;
                 }
@@ -337,7 +328,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
     private static final MailField[] FIELDS_ID = new MailField[] { MailField.ID };
 
-    private SavePointAndReason exportMessages(String fullName, String path, String startMailId, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference) throws OXException, DataExportAbortedException {
+    private SavePointAndReason exportMessages(String fullName, String path, String startMailId, MailOperationExecutor mailOperationExecutor) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
             JSONObject jSavePoint = new JSONObject(4);
             jSavePoint.putSafe("folder", fullName).putSafe("path", path);
@@ -350,7 +341,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
         MailMessage[] messages;
         try {
-            messages = mailAccessReference.getValue().getMessageStorage().getAllMessages(fullName, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, FIELDS_ID);
+            messages = mailOperationExecutor.executeWithRetryOnConnectionLoss(mailAccess -> mailAccess.getMessageStorage().getAllMessages(fullName, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, FIELDS_ID));
             if (messages.length <= 0) {
                 // No messages in given folder
                 return null;
@@ -387,7 +378,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 }
             }
 
-            LOG.debug("Going to export {} mails from directory {} for data export {} of user {} in context {}", I(messages.length), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+            LOG.debug("Going to export {} mails from directory \"{}\" for data export {} of user {} in context {}", I(messages.length), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
         } catch (DataExportAbortedException e) {
             throw e;
         } catch (Exception e) {
@@ -399,49 +390,19 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 return savePointFor(jSavePoint, e);
             }
             if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
-                LOG.debug("Failed to export messages from root folder from primary mail account of user {} in context {}", I(task.getUserId()), I(task.getContextId()), e);
+                LOG.debug("Failed to export messages from root directory from primary mail account for data export {} of user {} in context {}", UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
             } else {
                 if (isPermissionDenied(e)) {
-                    LOG.debug("Forbidden to export messages from folder \"{}\" from primary mail account of user {} in context {}", fullName, I(task.getUserId()), I(task.getContextId()), e);
+                    LOG.debug("Forbidden to export messages from directory \"{}\" from primary mail account for data export {} of user {} in context {}", fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
                     sink.addToReport(Message.builderWithPermissionDeniedType().appendToMessage("Insufficient permissions to export messages from folder \"").appendToMessage(fullName).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_MAIL).withTimeStamp(new Date()).build());
                 } else {
-                    LOG.warn("Failed to export messages from folder \"{}\" from primary mail account of user {} in context {}", fullName, I(task.getUserId()), I(task.getContextId()), e);
+                    LOG.warn("Failed to export messages from directory \"{}\" from primary mail account for data export {} of user {} in context {}", fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
                     sink.addToReport(Message.builder().appendToMessage("Failed to export messages from folder \"").appendToMessage(fullName).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_MAIL).withTimeStamp(new Date()).build());
                 }
             }
             return null;
         }
 
-        ServiceLookup services = this.services;
-        CheckedConsumer<Throwable> onRetryConsumer = new CheckedConsumer<Throwable>() {
-
-            @Override
-            public void accept(Throwable t) throws Exception {
-                MailService mailService = services.getServiceSafe(MailService.class);
-
-                MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess = null;
-                try {
-                    // Grab session associated with current mail access in order to establish a new one
-                    Session session = mailAccessReference.getValue().getSession();
-
-                    // Drop old mail access
-                    mailAccessReference.getValue().setWaiting(false);
-                    MailAccess.closeInstance(mailAccessReference.getAndSetValue(null), false);
-
-                    // Establish new mail access
-                    mailAccess = mailService.getMailAccess(session, 0);
-                    mailAccess.connect();
-                    mailAccess.setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
-                    mailAccessReference.setValue(mailAccess);
-                    mailAccess = null; // Avoid premature closing
-                } finally {
-                    MailAccess.closeInstance(mailAccess);
-                }
-
-                LOG.debug("Connection to mail server closed. Retrying...", t);
-            }
-        };
-
         int batchCount = 0;
         for (MailMessage message : messages) {
             String mailId = message.getMailId();
@@ -462,20 +423,20 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
             InputStream stream = null;
             try {
-                StreamFromMailPartCallable streamFromMailPartCallable = new StreamFromMailPartCallable(messages.length, batchCount, mailAccessReference, mailId, fullName, task);
-                stream = executeWithRetryOnConnectionClosure(streamFromMailPartCallable, onRetryConsumer);
+                StreamFromMailPartOperation streamFromMailPartCallable = new StreamFromMailPartOperation(messages.length, batchCount, mailId, fullName, task);
+                stream = mailOperationExecutor.executeWithRetryOnConnectionLoss(streamFromMailPartCallable);
                 if (stream != null) {
                     boolean exported = sink.export(stream, new Item(path, sanitizeNameForZipEntry(mailId + ".eml"), streamFromMailPartCallable.getSentDate()));
                     if (!exported) {
                         return savePointFor(new JSONObject(4).putSafe("folder", fullName).putSafe("path", path).putSafe("id", mailId));
                     }
-                    LOG.debug("Exported mail {} ({} of {}) from directory {} for data export {} of user {} in context {}", mailId, I(batchCount), I(messages.length), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                    LOG.debug("Exported mail {} ({} of {}) from directory \"{}\" for data export {} of user {} in context {}", mailId, I(batchCount), I(messages.length), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
                 }
             } catch (Exception e) {
                 if (isRetryableExceptionAndMayFail(e, sink)) {
                     return savePointFor(new JSONObject(4).putSafe("folder", fullName).putSafe("path", path).putSafe("id", mailId), e);
                 }
-                LOG.warn("Failed to export message {} in folder \"{}\" from primary mail account of user {} in context {}", mailId, fullName, I(task.getUserId()), I(task.getContextId()), e);
+                LOG.warn("Failed to export message {} from directory \"{}\" of primary mail account for data export {} of user {} in context {}", mailId, fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()), e);
                 sink.addToReport(Message.builder().appendToMessage("Failed to export message \"").appendToMessage(mailId).appendToMessage("\" in folder \"").appendToMessage(fullName).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_MAIL).withTimeStamp(new Date()).build());
             } finally {
                 Streams.close(stream);
@@ -485,31 +446,29 @@ public class MailDataExport extends AbstractDataExportProviderTask {
         return null;
     }
 
-    private static class StreamFromMailPartCallable implements Callable<InputStream> {
+    private static class StreamFromMailPartOperation implements MailOperation<InputStream> {
 
         private final int messagesLength;
         private final int currentBatchCount;
         private final String mailId;
         private final String fullName;
         private final DataExportTask task;
-        private final Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference;
         private Date sentDate;
 
-        StreamFromMailPartCallable(int messagesLength, int currentBatchCount, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference, String mailId, String fullName, DataExportTask task) {
+        StreamFromMailPartOperation(int messagesLength, int currentBatchCount, String mailId, String fullName, DataExportTask task) {
             super();
             this.messagesLength = messagesLength;
             this.currentBatchCount = currentBatchCount;
-            this.mailAccessReference = mailAccessReference;
             this.mailId = mailId;
             this.fullName = fullName;
             this.task = task;
         }
 
         @Override
-        public InputStream call() throws OXException {
-            MailMessage m = mailAccessReference.getValue().getMessageStorage().getMessage(fullName, mailId, false);
+        public InputStream execute(MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess) throws OXException {
+            MailMessage m = mailAccess.getMessageStorage().getMessage(fullName, mailId, false);
             if (m == null) {
-                LOG.debug("No such mail {} ({} of {}) in directory {} for data export {} of user {} in context {}", mailId, I(currentBatchCount), I(messagesLength), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                LOG.debug("No such mail {} ({} of {}) in directory \"{}\" for data export {} of user {} in context {}", mailId, I(currentBatchCount), I(messagesLength), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
                 return null;
             }
 
@@ -527,55 +486,6 @@ public class MailDataExport extends AbstractDataExportProviderTask {
         }
     }
 
-    /**
-     * Executes given callable with one-time retry behavior on connect error.
-     *
-     * @param callable The callable to execute
-     * @param onRetryConsumer The retry consumer
-     * @return The return value
-     * @throws OXException If a non-retriable exception occurs or retry attempts are expired
-     */
-    static InputStream executeWithRetryOnConnectionClosure(StreamFromMailPartCallable callable, CheckedConsumer<Throwable> onRetryConsumer) throws OXException {
-        try {
-            return Failsafe.with(RETRY_POLICY_CONNECTION_CLOSED).onRetry(onRetryConsumer).get(callable);
-        } catch (FailsafeException e) {
-            // Checked exception occurred
-            Throwable cause = e.getCause();
-            if (cause instanceof OXException) {
-                throw (OXException) cause;
-            }
-
-            Throwable t = cause == null ? e : cause;
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(t, t.getMessage());
-        } catch (IllegalArgumentException e) {
-            String message = e.getMessage();
-            if (message != null && message.startsWith("Invalid fully-qualifying mail folder identifier: ")) {
-                // Apparently, passed folder identifier is invalid
-                throw MailExceptionCode.INVALID_FOLDER_IDENTIFIER.create(e, message.substring(49));
-            }
-            // Unchecked exception occurred
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
-        } catch (RuntimeException e) {
-            // Unchecked exception occurred
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
-        }
-    }
-
-    private static final RetryPolicy RETRY_POLICY_CONNECTION_CLOSED = new RetryPolicy()
-        .withMaxRetries(1).withBackoff(1, 10, TimeUnit.SECONDS).withJitter(0.25f)
-        .retryOn(t ->
-            {
-                if (!(OXException.class.isInstance(t))) {
-                    return false;
-                }
-
-                OXException oxe = (OXException) t;
-                return MimeMailExceptionCode.CONNECTION_CLOSED.equals(oxe)
-                    || MimeMailExceptionCode.CONNECT_ERROR.equals(oxe)
-                    || MimeMailExceptionCode.READ_TIMEOUT.equals(oxe);
-            }
-        );
-
     // ------------------------------------------------------------ Helpers -------------------------------------------------------------
 
     private GeneratedSession generateSession(Module mailModule, SessionGenerator sessionGenerator) throws OXException {
