@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -29,6 +29,7 @@ import static com.openexchange.gdpr.dataexport.provider.general.AttachmentLoader
 import static com.openexchange.gdpr.dataexport.provider.general.SavePointAndReason.savePointFor;
 import static com.openexchange.java.Autoboxing.I;
 import static com.openexchange.java.Strings.parsePositiveInt;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -40,6 +41,7 @@ import java.util.Optional;
 import java.util.TimeZone;
 import org.json.JSONException;
 import org.json.JSONObject;
+import com.openexchange.ajax.container.ThresholdFileHolder;
 import com.openexchange.ajax.fileholder.IFileHolder;
 import com.openexchange.api2.TasksSQLInterface;
 import com.openexchange.context.ContextService;
@@ -78,16 +80,12 @@ import com.openexchange.groupware.tasks.TasksSQLImpl;
 import com.openexchange.i18n.Translator;
 import com.openexchange.i18n.TranslatorFactory;
 import com.openexchange.i18n.tools.StringHelper;
-import com.openexchange.java.BlockingAtomicReference;
-import com.openexchange.java.ExceptionAwarePipedInputStream;
-import com.openexchange.java.ExceptionForwardingPipedOutputStream;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
 import com.openexchange.notification.service.FullNameBuilderService;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
-import com.openexchange.threadpool.AbstractTask;
 import com.openexchange.threadpool.ThreadPoolService;
 import com.openexchange.tools.TimeZoneUtils;
 import com.openexchange.tools.iterator.SearchIterator;
@@ -105,6 +103,11 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(TaskDataExport.class);
 
+    private static final String JSON_FIELD_ID = "id";
+    private static final String JSON_FIELD_ROOT = "root";
+    private static final String JSON_FIELD_PATH = "path";
+    private static final String JSON_FIELD_FOLDER = "folder";
+
     private static final int PRIVATE_ID = FolderObject.SYSTEM_PRIVATE_FOLDER_ID;
     private static final int PUBLIC_ID = FolderObject.SYSTEM_PUBLIC_FOLDER_ID;
     private static final int SHARED_ID = FolderObject.SYSTEM_SHARED_FOLDER_ID;
@@ -144,18 +147,15 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
 
             NeededServices neededServices = new NeededServices(folderService, icalEmitter, contextService, threadPool);
 
-            Options options;
-            {
-                Module tasksModule = getModule();
-                boolean includePublicFolders = getBoolOption(TasksDataExportPropertyNames.PROP_INCLUDE_PUBLIC_FOLDERS, false, tasksModule);
-                boolean includeSharedFolders = getBoolOption(TasksDataExportPropertyNames.PROP_INCLUDE_SHARED_FOLDERS, false, tasksModule);
-                options = new Options(includePublicFolders, includeSharedFolders);
-            }
+            Module tasksModule = getModule();
+            boolean includePublicFolders = getBoolOption(TasksDataExportPropertyNames.PROP_INCLUDE_PUBLIC_FOLDERS, false, tasksModule);
+            boolean includeSharedFolders = getBoolOption(TasksDataExportPropertyNames.PROP_INCLUDE_SHARED_FOLDERS, false, tasksModule);
+            Options options = new Options(includePublicFolders, includeSharedFolders);
 
             if (savepoint.isPresent()) {
                 JSONObject jSavePoint = savepoint.get();
-                int taskId = jSavePoint.optInt("id", 0);
-                startInfo = new StartInfo(taskId > 0 ? I(taskId) : null, jSavePoint.getInt("folder"), jSavePoint.optString("path", null), jSavePoint.getInt("root"));
+                int taskId = jSavePoint.optInt(JSON_FIELD_ID, 0);
+                startInfo = new StartInfo(taskId > 0 ? I(taskId) : null, jSavePoint.getInt(JSON_FIELD_FOLDER), jSavePoint.optString(JSON_FIELD_PATH, null), jSavePoint.getInt(JSON_FIELD_ROOT));
             } else {
                 startInfo = null;
             }
@@ -231,17 +231,17 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
     private SavePointAndReason traverseFolder(int root, Type type, Folder folder, String parentPath, Options options, DecoratorProvider decoratorProvider, Session session, NeededServices neededServices) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
             if (startInfo != null) {
-                JSONObject jSavePoint = new JSONObject(4).putSafe("folder", startInfo.folderId).putSafe("root", I(root));
+                JSONObject jSavePoint = new JSONObject(4).putSafe(JSON_FIELD_FOLDER, startInfo.folderId).putSafe(JSON_FIELD_ROOT, I(root));
                 if (startInfo.path != null) {
-                    jSavePoint.putSafe("path", startInfo.path);
+                    jSavePoint.putSafe(JSON_FIELD_PATH, startInfo.path);
                 }
                 if (startInfo.taskId != null) {
-                    jSavePoint.putSafe("id", startInfo.taskId);
+                    jSavePoint.putSafe(JSON_FIELD_ID, startInfo.taskId);
                 }
                 return savePointFor(jSavePoint);
             }
 
-            return savePointFor(new JSONObject(2).putSafe("folder", folder.getFolderId()).putSafe("root", I(root)));
+            return savePointFor(new JSONObject(2).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_ROOT, I(root)));
         }
         checkAborted();
 
@@ -249,11 +249,11 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
         StartInfo info = startInfo;
         if (info == null || info.folderId.equals(folder.getFolderId())) {
             if (info == null || info.path == null) {
-                pathOfFolder = exportFolder(folder, parentPath);
+                pathOfFolder = exportFolderInternal(folder, parentPath);
                 if (pathOfFolder == null) {
-                    return savePointFor(new JSONObject(2).putSafe("folder", folder.getFolderId()).putSafe("root", I(root)));
+                    return savePointFor(new JSONObject(2).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_ROOT, I(root)));
                 }
-                LOG.debug("Exported task directory {} for data export {} of user {} in context {}", folder.getName(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                LOG.debug("Exported task directory {} for data export {} of user {} in context {}", folder.getName(), UUIDs.getUnformattedStringObjectFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
             } else {
                 pathOfFolder = info.path;
             }
@@ -273,7 +273,7 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
             List<Folder> children;
             try {
                 FolderService folderService = neededServices.folderService;
-                UserizedFolder[] visibleFolders = folderService.getVisibleFolders(folder.getFolderId(), FolderStorage.REAL_TREE_ID, decoratorProvider.contentType, type, true, session, decoratorProvider.createDecorator()).getResponse();
+                UserizedFolder[] visibleFolders = folderService.getVisibleFolders(FolderStorage.REAL_TREE_ID, decoratorProvider.contentType, type, true, session, decoratorProvider.createDecorator()).getResponse();
                 if (null == visibleFolders || visibleFolders.length <= 0) {
                     children = Collections.emptyList();
                 } else {
@@ -284,28 +284,32 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
 
                     children = new ArrayList<>(visibleFolders.length);
                     for (UserizedFolder subfolder : visibleFolders) {
+                        if (subfolder.getID().equals(folder.getFolderId())) {
+                            // Skip parent folder to prevent endless recursion
+                            continue;
+                        }
                         String namePrefix = null;
                         if (translator != null) {
                             FullNameBuilderService fullNameBuilder = services.getServiceSafe(FullNameBuilderService.class);
                             namePrefix = generateFullNamePrefix(fullNameBuilder.buildFullName(subfolder.getCreatedBy(), task.getContextId(), translator));
                         }
-                        children.add(new Folder(subfolder.getID(), namePrefix == null ? subfolder.getLocalizedName(locale) : namePrefix + subfolder.getLocalizedName(locale), false));
+                        children.add(new Folder(subfolder.getID(), namePrefix == null ? subfolder.getName(locale) : namePrefix + subfolder.getName(locale), false));
                     }
                 }
             } catch (Exception e) {
                 if (isRetryableExceptionAndMayFail(e, sink)) {
                     if (info != null) {
-                        JSONObject jSavePoint = new JSONObject(4).putSafe("folder", info.folderId).putSafe("root", I(root));
+                        JSONObject jSavePoint = new JSONObject(4).putSafe(JSON_FIELD_FOLDER, info.folderId).putSafe(JSON_FIELD_ROOT, I(root));
                         if (startInfo.path != null) {
-                            jSavePoint.putSafe("path", startInfo.path);
+                            jSavePoint.putSafe(JSON_FIELD_PATH, startInfo.path);
                         }
                         if (info.taskId != null) {
-                            jSavePoint.putSafe("id", info.taskId);
+                            jSavePoint.putSafe(JSON_FIELD_ID, info.taskId);
                         }
                         return savePointFor(jSavePoint, e);
                     }
 
-                    return savePointFor(new JSONObject(2).putSafe("folder", folder.getFolderId()).putSafe("root", I(root)), e);
+                    return savePointFor(new JSONObject(2).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_ROOT, I(root)), e);
                 }
                 LOG.warn("Failed to retrieve subfolders of folder \"{}\" from user {} in context {}", folder.getName(), I(task.getUserId()), I(task.getContextId()), e);
                 sink.addToReport(Message.builder().appendToMessage("Failed to retrieve subfolders of folder \"").appendToMessage(folder.getName()).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_TASKS).withTimeStamp(new Date()).build());
@@ -328,29 +332,21 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
         return null;
     }
 
-    private String exportFolder(Folder folderInfo, String path) throws OXException {
+    private String exportFolderInternal(Folder folderInfo, String path) throws OXException {
         return sink.export(new Directory(path, sanitizeNameForZipEntry(folderInfo.getName())));
     }
 
-    private static final Date DATE_ZERO = new Date(0);
+    private static final Date DATE_ZERO = new Date(0); // NOSONARLINT
 
     private static final int[] FIELDS_ID = new int[] { DataObject.OBJECT_ID };
 
-    private static final Comparator<Task> COMPARATOR = new Comparator<Task>() {
-
-        @Override
-        public int compare(Task o1, Task o2) {
-            int x = o1.getObjectID();
-            int y = o2.getObjectID();
-            return (x < y) ? -1 : ((x == y) ? 0 : 1);
-        }
-    };
+    private static final Comparator<Task> COMPARATOR = (task1, task2) -> Integer.compare(task1.getObjectID(), task2.getObjectID());
 
     private SavePointAndReason exportTasks(int root, Folder folder, String path, Integer startTaskId, Session session, NeededServices neededServices) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
-            JSONObject jSavePoint = new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", I(root));
+            JSONObject jSavePoint = new JSONObject(4).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_PATH, path).putSafe(JSON_FIELD_ROOT, I(root));
             if (startTaskId != null) {
-                jSavePoint.putSafe("id", startTaskId);
+                jSavePoint.putSafe(JSON_FIELD_ID, startTaskId);
             }
             return savePointFor(jSavePoint);
         }
@@ -373,10 +369,10 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
 
             if (isPauseRequested()) {
                 JSONObject jSavePoint = new JSONObject(4);
-                jSavePoint.putSafe("folder", folder.getFolderId());
-                jSavePoint.putSafe("path", path);
-                jSavePoint.putSafe("root", I(root));
-                jSavePoint.putSafe("id", startTaskId != null ? startTaskId : I(queriedTasks.next().getObjectID()));
+                jSavePoint.putSafe(JSON_FIELD_FOLDER, folder.getFolderId());
+                jSavePoint.putSafe(JSON_FIELD_PATH, path);
+                jSavePoint.putSafe(JSON_FIELD_ROOT, I(root));
+                jSavePoint.putSafe(JSON_FIELD_ID, startTaskId != null ? startTaskId : I(queriedTasks.next().getObjectID()));
                 return savePointFor(jSavePoint);
             }
             checkAborted();
@@ -401,14 +397,14 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
                 }
             }
 
-            LOG.debug("Going to export {} tasks from directory {} for data export {} of user {} in context {}", I(tasks.size()), folder.getName(), UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+            LOG.debug("Going to export {} tasks from directory {} for data export {} of user {} in context {}", I(tasks.size()), folder.getName(), UUIDs.getUnformattedStringObjectFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
         } catch (DataExportAbortedException e) {
             throw e;
         } catch (Exception e) {
             if (isRetryableExceptionAndMayFail(e, sink)) {
-                JSONObject jSavePoint = new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", I(root));
+                JSONObject jSavePoint = new JSONObject(4).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_PATH, path).putSafe(JSON_FIELD_ROOT, I(root));
                 if (startTaskId != null) {
-                    jSavePoint.putSafe("id", startTaskId);
+                    jSavePoint.putSafe(JSON_FIELD_ID, startTaskId);
                 }
                 return savePointFor(jSavePoint, e);
             }
@@ -427,94 +423,100 @@ public class TaskDataExport extends AbstractDataExportProviderTask {
         int batchCount = 0;
         List<ConversionError> errors = new LinkedList<>();
         List<ConversionWarning> warnings = new LinkedList<>();
-        for (Task task : tasks) {
-            int taskId = task.getObjectID();
+        ThresholdFileHolder fileHolder = null;
+        try {
+            for (Task task : tasks) {
+                int taskId = task.getObjectID();
 
-            if (isPauseRequested()) {
-                JSONObject jSavePoint = new JSONObject(4);
-                jSavePoint.putSafe("folder", folder.getFolderId());
-                jSavePoint.putSafe("path", path);
-                jSavePoint.putSafe("root", I(root));
-                jSavePoint.putSafe("id", I(taskId));
-                return savePointFor(jSavePoint);
-            }
-            int count = incrementAndGetCount();
-            checkAborted((count % 100 == 0));
-            if (count % 1000 == 0) {
-                sink.setSavePoint(new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", I(root)).putSafe("id", I(taskId)));
-            }
-            batchCount++;
-
-            List<IFileHolder> attachments = null;
-            try {
-                // Query full task
-                Task t = tasksSql.getTaskById(taskId, iFolderId);
-
-                // Prepare attachments
-                if (t.getNumberOfAttachments() > 0) {
-                    int folderId = Strings.getUnsignedInt(folder.getFolderId());
-                    if (folderId > 0) {
-                        Optional<List<IFileHolder>> optionalAttachments = loadAttachmentBinaries(taskId, com.openexchange.groupware.Types.TASK, folderId, session);
-                        if (optionalAttachments != null && optionalAttachments.isPresent()) {
-                            attachments = optionalAttachments.get();
-                            t.setProperty(PROPERTY_BINARY_ATTACHMENTS, attachments);
-                        }
-                    }
+                if (isPauseRequested()) {
+                    JSONObject jSavePoint = new JSONObject(4);
+                    jSavePoint.putSafe(JSON_FIELD_FOLDER, folder.getFolderId());
+                    jSavePoint.putSafe(JSON_FIELD_PATH, path);
+                    jSavePoint.putSafe(JSON_FIELD_ROOT, I(root));
+                    jSavePoint.putSafe(JSON_FIELD_ID, I(taskId));
+                    return savePointFor(jSavePoint);
                 }
+                int count = incrementAndGetCount();
+                checkAborted((count % 100 == 0));
+                if (count % 1000 == 0) {
+                    sink.setSavePoint(new JSONObject(4).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_PATH, path).putSafe(JSON_FIELD_ROOT, I(root)).putSafe(JSON_FIELD_ID, I(taskId)));
+                }
+                batchCount++;
 
-                // Write task as iCal to iCal session
-                ICalSession iCalSession = icalEmitter.createSession();
-                icalEmitter.writeTask(iCalSession, t, context, errors, warnings);
-
-                // Initiate piped streams to pipe iCal output to sink
-                ExceptionAwarePipedInputStream in = new ExceptionAwarePipedInputStream(65536);
-                ExceptionForwardingPipedOutputStream out = new ExceptionForwardingPipedOutputStream(in);
+                List<IFileHolder> attachments = null;
                 try {
-                    BlockingAtomicReference<Boolean> exportedFlag = new BlockingAtomicReference<>();
-                    DataExportSink sink = this.sink;
-                    AbstractTask<Void> fileStorageWriter = new AbstractTask<Void>() {
+                    // Query full task
+                    Task t = tasksSql.getTaskById(taskId, iFolderId);
 
-                        @Override
-                        public Void call() throws OXException {
-                            boolean exported = sink.export(in, new Item(path, taskId + ".ics", null));
-                            exportedFlag.set(Boolean.valueOf(exported));
-                            return null;
+                    // Prepare attachments
+                    if (t.getNumberOfAttachments() > 0) {
+                        int folderId = Strings.getUnsignedInt(folder.getFolderId());
+                        if (folderId > 0) {
+                            Optional<List<IFileHolder>> optionalAttachments = loadAttachmentBinaries(taskId, com.openexchange.groupware.Types.TASK, folderId, session);
+                            if (optionalAttachments.isPresent()) {
+                                attachments = optionalAttachments.get();
+                                t.setProperty(PROPERTY_BINARY_ATTACHMENTS, attachments);
+                            }
                         }
-                    };
-                    threadPool.submit(fileStorageWriter);
-                    icalEmitter.writeSession(iCalSession, out);
-                    Boolean exported = exportedFlag.getUninterruptibly();
-                    if (!exported.booleanValue()) {
-                        return savePointFor(new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", I(root)).putSafe("id", I(taskId)));
                     }
-                    LOG.debug("Exported task {} ({} of {}) from directory {} for data export {} of user {} in context {}", I(taskId), I(batchCount), I(tasks.size()), folder.getName(), UUIDs.getUnformattedString(this.task.getId()), I(this.task.getUserId()), I(this.task.getContextId()));
+
+                    // Write task as iCal to iCal session
+                    ICalSession iCalSession = icalEmitter.createSession();
+                    icalEmitter.writeTask(iCalSession, t, context, errors, warnings);
+
+                    // Write iCal
+                    InputStream iCalContent = null;
+                    try {
+                        // Write iCal content to file holder
+                        fileHolder = initOrReset(fileHolder);
+                        icalEmitter.writeSession(iCalSession, fileHolder.asOutputStream());
+
+                        if (fileHolder.getLength() <= 0) {
+                            // No bytes written
+                            LOG.debug("No export for task {} ({} of {}) from directory {} for data export {} of user {} in context {}. Reason: Empty iCal written", I(taskId), I(batchCount), I(tasks.size()), folder.getName(), UUIDs.getUnformattedStringObjectFor(this.task.getId()), I(this.task.getUserId()), I(this.task.getContextId()));
+                        } else {
+                            // Export file holder's content
+                            iCalContent = fileHolder.getStream();
+                            boolean exported = sink.export(iCalContent, new Item(path, taskId + ".ics", null));
+                            if (!exported) {
+                                return savePointFor(new JSONObject(4).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_PATH, path).putSafe(JSON_FIELD_ROOT, I(root)).putSafe(JSON_FIELD_ID, I(taskId)));
+                            }
+                            LOG.debug("Exported task {} ({} of {}) from directory {} for data export {} of user {} in context {}", I(taskId), I(batchCount), I(tasks.size()), folder.getName(), UUIDs.getUnformattedStringObjectFor(this.task.getId()), I(this.task.getUserId()), I(this.task.getContextId()));
+                        }
+                    } finally {
+                        Streams.close(iCalContent);
+                    }
                 } catch (Exception e) {
-                    Throwable cause = e.getCause();
-                    if (null == cause) {
-                        in.setException(e);
-                    } else if (cause instanceof Exception) {
-                        in.setException((Exception) cause);
-                    } else {
-                        in.setException(new Exception(e));
+                    if (isRetryableExceptionAndMayFail(e, sink)) {
+                        return savePointFor(new JSONObject(4).putSafe(JSON_FIELD_FOLDER, folder.getFolderId()).putSafe(JSON_FIELD_PATH, path).putSafe(JSON_FIELD_ROOT, I(root)).putSafe(JSON_FIELD_ID, I(taskId)), e);
                     }
-                    throw e;
+                    LOG.warn("Failed to export task {} in folder \"{}\" for data export {} of user {} in context {}", I(taskId), folder.getName(), UUIDs.getUnformattedStringObjectFor(this.task.getId()), I(this.task.getUserId()), I(this.task.getContextId()), e);
+                    sink.addToReport(Message.builder().appendToMessage("Failed to export task \"").appendToMessage(taskId).appendToMessage("\" in folder \"").appendToMessage(folder.getName()).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_TASKS).withTimeStamp(new Date()).build());
                 } finally {
-                    Streams.close(in, out);
+                    Streams.close(attachments);
                 }
-            } catch (Exception e) {
-                if (isRetryableExceptionAndMayFail(e, sink)) {
-                    return savePointFor(new JSONObject(4).putSafe("folder", folder.getFolderId()).putSafe("path", path).putSafe("root", I(root)).putSafe("id", I(taskId)), e);
-                }
-                LOG.warn("Failed to export task {} in folder \"{}\" from primary mail account of user {} in context {}", I(taskId), folder.getName(), I(this.task.getUserId()), I(this.task.getContextId()), e);
-                sink.addToReport(Message.builder().appendToMessage("Failed to export task \"").appendToMessage(taskId).appendToMessage("\" in folder \"").appendToMessage(folder.getName()).appendToMessage("\": ").appendToMessage(e.getMessage()).withModuleId(ID_TASKS).withTimeStamp(new Date()).build());
-            } finally {
-                Streams.close(attachments);
             }
+        } finally {
+            Streams.close(fileHolder);
         }
 
         return null;
     }
 
+    /**
+     * Initializes a new file holder if passed reference is <code>null</code>; otherwise given instance is reset and returned.
+     *
+     * @param fileHolder The currently used file holder instance or <code>null</code>
+     * @return The file holder to use
+     */
+    private static ThresholdFileHolder initOrReset(ThresholdFileHolder fileHolder) {
+        if (fileHolder == null) {
+            return new ThresholdFileHolder(false);
+        }
+        fileHolder.reset();
+        return fileHolder;
+    }
+
     // ------------------------------------------------------------ Helpers -------------------------------------------------------------
 
     private static class Folder extends com.openexchange.gdpr.dataexport.provider.general.Folder {
